from django.contrib import admin
from django.utils.safestring import mark_safe

from carbon.atoms.admin.content import *
from carbon.atoms.admin.taxonomy import *

from reversion.admin import VersionAdmin
from django_inline_wrestler.admin import TabularInlineOrderable

from carbon.atoms.admin.content import BaseVersionableAdmin, BaseVersionableTitleAdmin

# from .models import *
from .forms import *



class EmailTemplateAdmin(VersionAdmin, BaseVersionableTitleAdmin):
    
    form = EmailTemplateAdminForm
    prepopulated_fields = {"slug": ("title",)}

    core_fields = (
        ('title','slug'),
        'subject_template',
        'body_template',
    )
    meta_fields = BaseVersionableAdmin.meta_fields
    fieldsets = (
        ("Template", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )    

    search_fields = ('title','admin_note', 'subject_template', 'body_template')


class EmailCategoryAdmin(VersionAdmin, BaseVersionableTitleAdmin):

    prepopulated_fields = {"slug": ("title",)}

    autocomplete_lookup_fields = {
        'fk': ('parent',),
        'm2m': ()
    }
    raw_id_fields = ('parent',)

    core_fields = (
        ('edit_parent','parent',),
        ('title','slug'),
        'can_be_viewed_online',
        'requires_explicit_opt_in',
        'can_unsubscribe'
    )
    meta_fields = BaseVersionableAdmin.meta_fields
    fieldsets = (
        ("Category", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )
    readonly_fields = BaseVersionableAdmin.readonly_fields + ('edit_parent',)
    list_display = ('parent', 'title', 'can_be_viewed_online', 'requires_explicit_opt_in','can_unsubscribe')
    list_display_links = ('parent','title')
    list_filter = ('parent','can_be_viewed_online','requires_explicit_opt_in','can_unsubscribe')


class EmailReceiptAdmin(VersionAdmin, BaseVersionableAdmin):

    def rendered_html_iframe(self, obj):
        url = obj.get_rendered_url()
        return mark_safe('<iframe src="%s" style="width: 700px; height: 800px" ></iframe>'%( url ))
    

    autocomplete_lookup_fields = {
        'fk': ('category'),
        'm2m': ()
    }
    raw_id_fields = ( 'category',)

    core_fields = (
        ('recipient_email','category'),
        
    )
    
    stats_fields = (
        ('access_key',),
        ('viewed','view_count'),
        ('viewed_from_email','viewed_online'),
        ('first_viewed_date','last_viewed_date'),
    )
    rendered_fields = (
        ('rendered_subject'),
        ('rendered_html_iframe'),        
        ('rendered_body'),
        'sending_error',
        ('sending_error_message')
    )
    meta_fields = BaseVersionableAdmin.meta_fields
    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Stats", {
            'fields': stats_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Rendered", {
            'fields': rendered_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )
    list_display = ('recipient_email','category','rendered_subject','sending_error','viewed',
        'view_count','created_date','first_viewed_date')
    list_filter = ('recipient_email','category','viewed', 'sending_error', )
    readonly_fields = BaseVersionableAdmin.readonly_fields + ('access_key',
        'rendered_body','rendered_subject','rendered_html_iframe',
        'sending_error','sending_error_message')



class EmailCategorySubscriptionSettingsAdmin(VersionAdmin, BaseVersionableTitleAdmin):

    # prepopulated_fields = {"slug": ("title",)}

    autocomplete_lookup_fields = {
        'fk': ('category','parent',),
        'm2m': ()
    }
    raw_id_fields = ('category','parent',)

    core_fields = (
        ('parent','category'),
        'status'
    )
    meta_fields = BaseVersionableAdmin.meta_fields
    fieldsets = (
        ("Category", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )


class EmailCategorySubscriptionSettingsInline(admin.TabularInline):
    #model = EmailCategorySubscriptionSettings  


    fk_name = 'parent'
    autocomplete_lookup_fields = {
        'fk': ('category',),
        'm2m': ()
    }
    raw_id_fields = ('category',)

    fields = ('category','status')
    readonly_fields = ['category']
    
    extra = 0

class UserSubscriptionSettingsAdmin(VersionAdmin, BaseVersionableAdmin):

    

    core_fields = (
        ('recipient_email',),
        'access_key'
    )
    meta_fields = BaseVersionableAdmin.meta_fields
    fieldsets = (
        ("Category", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )
    list_display = ('recipient_email',)
    list_filter = ('recipient_email',)

    # inlines = [EmailCategorySubscriptionSettingsInline]




# admin.site.register(EmailTemplate, EmailTemplateAdmin)
# admin.site.register(EmailCategory, EmailCategoryAdmin)
# admin.site.register(EmailReceipt, EmailReceiptAdmin)
# admin.site.register(EmailCategorySubscriptionSettings, EmailCategorySubscriptionSettingsAdmin)
# admin.site.register(UserSubscriptionSettings, UserSubscriptionSettingsAdmin)


import logging
try:
	from PIL import Image
except ImportError:
	import Image

from django.conf import settings
from django.contrib.auth.decorators import user_passes_test
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse_lazy, reverse
from django.contrib import messages
from django.forms.formsets import formset_factory
from django.http import HttpResponse, HttpResponseRedirect, Http404, HttpResponseForbidden
from django.views.generic import DetailView, ListView
from django.utils.decorators import method_decorator
try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model

from carbon.atoms.views.abstract import *

from .models import *
from .forms import *


def user_is_admin(user):
	return user.is_staff

class EmailOnlineView(DetailView):
	slug_url_kwarg = 'access_key'
	slug_field = 'access_key'



class EmailRenderedView(DetailView):
	slug_url_kwarg = 'access_key'
	slug_field = 'access_key'

	@method_decorator(user_passes_test(lambda u: u.is_staff))
	def dispatch(self, *args, **kwargs):
		return super(EmailRenderedView, self).dispatch(*args, **kwargs)

	def render_to_response(self, context, **response_kwargs):            
		return HttpResponse(self.object.rendered_body, content_type="text/html")


class EmailSettingsView(DetailView):
	slug_url_kwarg = 'access_key'
	slug_field = 'access_key'
	
	def get(self, request, *args, **kwargs):

		self.object = self.get_object()      
		
		settings = self.object.get_settings()
		settings_data = [{'parent':item.parent.pk,'category':item.category.pk,'status':item.status} for item in settings]

		EmailCategorySubscriptionSettingsFormSet = formset_factory(EmailCategorySubscriptionSettingsForm, extra=0)
		self.formset = EmailCategorySubscriptionSettingsFormSet(initial=settings_data)
		
		context = self.get_context_data(object=self.object, formset=self.formset)
		return self.render_to_response(context)

	def post(self, request, *args, **kwargs):

		self.object = self.get_object()  

		EmailCategorySubscriptionSettingsFormSet = formset_factory(EmailCategorySubscriptionSettingsForm, extra=0)
		self.formset = EmailCategorySubscriptionSettingsFormSet(initial=settings_data)
		
		if self.forset.is_valid():        
			messages.success(request, 'Your subscription settings were updated')
		else:        
			messages.error(request, 'Please correct the errors below')

		context = self.get_context_data(object=self.object, formset=self.formset)
		return self.render_to_response(context)

	def put(self, *args, **kwargs):
		return self.post(*args, **kwargs)
	
class EmailRecordView( DetailView):
	slug_url_kwarg = 'access_key'
	slug_field = 'access_key'

	def render_to_response(self, context, **response_kwargs):  

		sites = Site.objects.all()
		from_any_internal_site = is_request_from_any_internal_site(self.request, sites)
		if from_any_internal_site == False:
			self.object.record_view()
		
		return output_spaceball_image()

def is_request_from_any_internal_site(request, sites):
	for site in sites:
		if request_originated_from_site(request, site) == True:
			return True
	return False

def is_request_from_any_online_view(request, sites, receipt):
	for site in sites:
		if request_originated_from_online_view(request, site, receipt) == True:
			return True
	return False    

def request_originated_from_site(request, site):
	if "HTTP_REFERER" in request.META:
		referer = request.META['HTTP_REFERER']
		if site.domain in referer:
			return True
	return False

def request_originated_from_online_view(request, site, receipt):
	if request_originated_from_site(request, site):
		online_url = receipt.get_rendered_url()
		referer = request.META['HTTP_REFERER']
		if online_url in referer:
			return True
	return False    

def output_spaceball_image():

	img = Image.new("RGB", (1,1), "#ffffff")
	response = HttpResponse(content_type="image/jpeg")
	img.save(response, "JPEG")
	return response




from django.conf import settings
from django.contrib.sitemaps import Sitemap

from .abstract import AddressibleSitemap

class PublishableSitemap(AddressibleSitemap):    

    def items(self):   
    	all_items = self.model.objects.published()
    	return [item for item in all_items if item.is_published()]

class SEOSitemap(PublishableSitemap):    

    def priority(self, obj):
        return obj.sitemap_priority

    def changefreq(self, obj):
        return obj.sitemap_changefreq        


# -*- coding: utf-8 -*-
import os
import errno
import re
import urllib2, urllib
import tempfile
import traceback
import sys
import shutil
import zipfile
import codecs
import csv
import httplib2
from bs4 import BeautifulSoup

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

from gzip import GzipFile

from boto.s3.key import Key
import boto.s3

from django.conf import settings
from django.core.urlresolvers import reverse
from django.core.files.base import ContentFile
from django.db import models
from django.dispatch import receiver
from django.template import Template as DjangoTemplate
from django.template import loader
from django.template.loaders.filesystem import Loader as FileSystemLoader
from django.utils.module_loading import import_string
from django.utils.text import slugify
from django.utils.text import Truncator
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType


#CSS/JS libs
from slimit import minify
from csscompressor import compress
import sass

from carbon.utils.slugify import unique_slugify
from carbon.utils.template import get_page_templates, get_page_templates_raw, get_all_templates

from carbon.atoms.models.abstract import *
from carbon.atoms.models.content import *



class LegacyURLReferer(VersionableAtom):

    help = {
        'legacy_url': "",
        'referer_title': "",
        'referer_url': "",
    }

   
    #legacy_url = models.ForeignKey('LegacyURL')

    referer_title = models.CharField(_('Referer Title'), max_length=255, 
        help_text=help['referer_title'], blank=True, null=True)

    referer_url = models.CharField(_('Referer URL'), max_length=255, 
        help_text=help['referer_url'], blank=True, null=True)


    class Meta:
        abstract = True

class LegacyURL(VersionableAtom, AddressibleAtom):

    help = {
        'url': "",
    }

    url = models.CharField(_("URL"), max_length = 255, blank = False,
        db_index=True)
   
    #Point to an object
    try:
        content_type = models.ForeignKey(ContentType, 
            limit_choices_to={"model__in": settings.LEGACY_URL_CHOICES}, 
            null=True, blank=True)
    except:
        content_type = models.ForeignKey(ContentType, null=True, blank=True)

    object_id = models.PositiveIntegerField(null=True, blank=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    
    class Meta:
        abstract = True        

    def get_children(self):
        return []

    @property
    def has_redirect_url(self):
        url = self.get_redirect_url()
        if url:
            return True
        return False

    def get_redirect_url(self):
        if self.path:
            return self.path
        return self.compute_get_redirect_url()  

    def generate_path(self):
        if self.path_override:
            return self.path_override
        if self.content_object:
            try:
                url = self.content_object.get_absolute_url()
                return url
            except:
                print "ERROR RETRIEVING ABSOLUTE URL From %s"%(self.content_object)         
        
        return None

    @staticmethod
    def autocomplete_search_fields():
        return ("id__iexact", "url__icontains", "title__icontains",)

    @classmethod
    def import_links(cls, file, request=None):

        try:
            ignore_files = settings.LEGACY_URL_IGNORE_LIST
            legacy_domain = settings.LEGACY_URL_ARCHIVE_DOMAIN
            legacy_domain_ssl = settings.LEGACY_URL_ARCHIVE_DOMAIN.replace("http://", "https://")
        except:
            ignore_files = []
            legacy_domain = None
            legacy_domain_ssl = None
        
        # print legacy_domain

        if legacy_domain or legacy_domain_ssl:
             # try:
                

            
            items_found = 0

            ctr = 0
            for row in csv.reader(file.read().splitlines()):

                #dont read from first row -- which contains column titles
                if ctr > 0:

                    status_check    = row[0].lower()
                    url             = row[1]
                    referer         = row[2]
                    
                    if legacy_domain in url or legacy_domain_ssl in url:
                        
                        is_allowed = True
                        for piece in ignore_files:
                            if piece in url:
                                is_allowed = False

                        if is_allowed:

                            items_found     += 1
                                                    
                            http = httplib2.Http()
                            status, response = http.request(url)
                            soup = BeautifulSoup(response)  
                            try:
                                page_title = soup.title.string.strip()
                            except:
                                page_title == "Untitled %s"%(url)
                                
                            # print "%s (%s)"%(url, page_title)
                            legacy_link = cls.create_legacy_url(url, page_title)
                            

                ctr += 1

            return '%s links parsed' % (items_found)
            # except Exception, e:
            #     return "There was an error reading the .csv file: %s"%(e)
        else:
            return "LEGACY_URL_ARCHIVE_DOMAIN not specified in settings"

    @classmethod
    def create_legacy_url(cls, target_url, target_name, referer_url=None, referer_title=None):
        
        path = cls.clean_path(target_url)
        
        if path:
            link, link_created = cls.objects.get_or_create(url=path)

            if link_created or path != target_name:
                link.title = target_name
                link.save()


            if referer_url:
                referer_link, referer_created = cls.objects.get_or_create(legacy_url=link,referer_url=referer_url)
                if referer_created:
                    if settings.DEBUG:
                        print "Create new referer %s %s to %s"%(referer_title, referer_url, target_url)
                    referer_link.referer_title = referer_title
                    referer_link.save()

            return link

        return None

    @classmethod
    def add_referer(cls, referer_url, referer_title, target_url):

        path = cls.clean_path(target_url)
        if path:
            legacy_link = cls.create_legacy_url(path, path, referer_url, referer_title)
            return legacy_link
        return None

    @classmethod
    def clean_path(cls, url):

        legacy_domain = settings.LEGACY_URL_ARCHIVE_DOMAIN
        legacy_domain_ssl = settings.LEGACY_URL_ARCHIVE_DOMAIN.replace("http://", "https://")

        path = url
        path = path.replace(legacy_domain, '')
        path = path.replace(legacy_domain_ssl, '')

        if 'http' in path:
            #print "Must be a different domain %s"%(path)
            return None

        if path == '' or path == None or path == '#':
            return None

        #Make sure path includes starting /
        if path.startswith('/') == False:
            path = '/%s'%path

        return path

########################################################################
## MENU GROUPS #########################################################
########################################################################


class MenuItem(VersionableAtom, HierarchicalAtom, LinkAtom, PublishableAtom, AddressibleAtom):
    
    publish_by_default = True

    help = {
    }

    
    #Point to an object
    try:
        content_type = models.ForeignKey(ContentType, 
            limit_choices_to={"model__in": settings.MENU_MODEL_CHOICES}, 
            null=True, blank=True)
    except:
        content_type = models.ForeignKey(ContentType, null=True, blank=True)

    object_id = models.PositiveIntegerField(null=True, blank=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    def generate_path(self):
        if self.content_object:
            if hasattr(self.content_object, 'get_absolute_url'):
                return self.content_object.get_absolute_url()
        
        return self.path_override

    def get_absolute_url(self):
        if self.content_object:
            if hasattr(self.content_object, 'get_absolute_url'):
                return self.content_object.get_absolute_url()
        
        return self.path_override


    def get_link(self):
        '<a href="%s" target="%s" class="%s" %s>%s</a>'%(self.get_path, self.target, self.css_classes, self.title, self.extra_attributes)


    def save(self, *args, **kwargs):

        #Use title if not specified
        if not self.title and self.content_object:
            try:
                self.title = self.content_object.title
            except:
                pass

        super(MenuItem, self).save(*args, **kwargs)

    
    class Meta:
        abstract = True
        ordering = ['order']
    


class AdminAppGroup( VersionableAtom, AddressibleAtom ):

    open_by_default = models.BooleanField(default=True)

    class Meta:
        abstract = True
        ordering = ['order']

    def get_children(self):
        if not self.item_class:
            raise NotImplementedError('Class should specify an item_class value')
        return self.item_class.objects.filter(parent=self).order_by('order')

class AdminAppLink(VersionableAtom, AddressibleAtom):

    parent = models.ForeignKey('AdminAppGroup', blank = True, null = True)

    help = {
        'model_path': "e.x. blog.models.BlogArticle",
    }

    model_path = models.CharField(_("Model Path"), max_length = 255, blank = False,
        db_index=True, help_text=help['model_path'])    



    class Meta:
        abstract = True

class AdminSidebar( VersionableAtom, AddressibleAtom ):
    class Meta:
        abstract = True

    def get_children(self):
        if not self.item_class:
            raise NotImplementedError('Class should specify an item_class value')
        return self.item_class.objects.filter(parent=self).order_by('order')

class AdminLink(VersionableAtom, AddressibleAtom):

    parent = models.ForeignKey('AdminSidebar', blank = True, null = True)

    help = {
        'url': "",
    }

    url = models.CharField(_("URL"), max_length = 255, blank = False,
        db_index=True, help_text=help['url'])      

    class Meta:
        abstract = True    



########################################################################
## TEMPLATES AND RESOURCES #############################################
########################################################################

class Template(VersionableAtom, TitleAtom):

    help = {
        'custom_template': "Override html template file with a custom template.",
        'template':"Choose an existing html template file. This will be overwritten in custom template is filled in.",
        'slug':"This slug can be referenced within templates: {% extends template-slug %}"
    }


    custom_template = models.TextField(_('custom template'), 
        help_text=help['custom_template'], null=True, blank=True)
    file_template = models.CharField(_('Template'), max_length=255, 
        choices=get_page_templates(), null=True, blank=True,
        help_text=help['template'])


    @staticmethod
    def autocomplete_search_fields():
        return ("admin_note__icontains","title__icontains")

    def get_content(self):
        if self.custom_template:
            return self.custom_template
        elif self.file_template:
            loader = FileSystemLoader(self.file_template)
            source = loader.load_template_source(self.file_template)
            return source[0]
        return ''
        
    def generate_slug(self):
        unique_slugify(self, self.title, 'slug', None)
        return self.slug


    def save(self, *args, **kwargs):

        template_content_changed = False
        original = None
        if self.pk is not None:
            model = type(self)
            original = model.objects.get(pk=self.pk)

        #Clear html template if custom content is defined
        if self.custom_template != None and self.custom_template != '':
            self.file_template = None

        if original != None and original.custom_template != self.custom_template:
            template_content_changed = True

        super(Template, self).save(*args, **kwargs)

        if template_content_changed == True:
            #If template has changed, clear cache key
            from django.core.cache import cache
            from carbon.utils.template import get_template_cache_key
            cache_key = get_template_cache_key(self.slug)
            cache.delete(cache_key)
        

    class Meta:
        ordering = ['title']
        abstract = True



def title_file_name( instance, filename ):
    
    subfolder = (instance.__class__.__name__).lower()

    file, extension = os.path.splitext( filename )    
    #if instance.clean_filename_on_upload:
    #filename     = "%s%s"%(slugify(filename[:245]), extension)
    filename     = filename.lower()

    full_path = '/'.join( [ subfolder, filename ] )

    try:
        #if instance.allow_overwrite==True:
        if hasattr(instance, 'file_source') and instance.file_source.storage.exists(full_path):
            instance.file_source.storage.delete(full_path)

        #if instance.allow_overwrite==True:
        if hasattr(instance, 'file_minified') and instance.file_minified.storage.exists(full_path):
            instance.file_minified.storage.delete(full_path)
    
    except Exception, err:
        print "Warning: Error deleting previous file contents"
        
    return full_path


def get_storage():
    storage = import_string(settings.MEDIA_STORAGE)()
    return storage

class BaseFrontendPackage(VersionableAtom, TitleAtom):

    help = {
        'peg_revision':"While making unstable changes, you may peg the package \
        to last working version number. Then newer packages will only be \
        returned if settings.DEBUG=True. Leave blank to use the lastest version.",
        'title':"File Name",
        'slug':"The filename for this item, without '.min' or the file extension",
        'source':"Copy the un-minified source",
        'minified':"Copy the minified source"
    }

    peg_revision = models.PositiveIntegerField(null=True, blank=True, help_text=help['peg_revision'])
    
    generated_file_source = models.TextField(null=True, blank=True)
    generated_file_minified = models.TextField(null=True, blank=True)
    error_source_content = models.TextField(null=True, blank=True)

    file_source = models.FileField(upload_to=title_file_name, blank=True, null=True, storage=get_storage())
    file_minified = models.FileField(upload_to=title_file_name, blank=True, null=True, storage=get_storage())

    needs_render = models.BooleanField(default=False)

    def get_children(self):
        if not self.item_class:
            raise NotImplementedError('Class should specify an item_class value')
        return self.item_class.objects.filter(parent=self).order_by('order')

    def get_extension(self):
        #Override in subclass
        return '.txt'

    def get_header_type(self):
        #Override in subclass
        return 'text/html'

    def get_src_directories(self):
        output = []
        children = self.get_children()
        for child in children:
            if child.is_source_package():
                output.append(child.get_package_source_folder())

        return output
           

    @staticmethod
    def autocomplete_search_fields():
        return ("admin_note__icontains","title__icontains",)

    def request_render(self):
        self.needs_render = True
        self.save()


    def render(self, save_file=True, force=False):
        # print 'render(save_file=%s, force=%s) needs_render? %s'%(save_file, force, self.needs_render)
        if force==False and self.needs_render==False and self.error_source_content==None:
            return

        source, success, error_message = self.get_source()

        if success:
            self.error_source_content = None

            try:
                minified_source = self.minify(source)
            except Exception, err:
                self.error_source_content = 'Error minifying %s: %s - %s'%(self.title, traceback.format_exc(), sys.exc_info()[0])
                minified_source = self.generated_file_minified

            is_different = self.generated_file_minified != minified_source

            if is_different or force==True:
                
                self.generated_file_minified = minified_source
                self.generated_file_source = source
                
                #Only increment version number on success, and if different
                self.version = self.version+1

                source = u'/* %s - v%s */%s'%(self.title, self.version, source)
                minified_source = u'/* %s - v%s */%s'%(self.title, self.version, minified_source)
                source_file = ContentFile(source)
                minified_file = ContentFile(minified_source)
                
                source_name = self.get_file_name()
                minified_name = self.get_file_name(True)

                self.file_source.save(source_name, source_file, save=save_file)
                self.file_minified.save(minified_name, minified_file, save=save_file)

                subfolder = self.__class__.__name__.lower()
                archived_source_path = '/%s/%s/archive/%s/%s'%(subfolder, self.slug, self.version, source_name)
                archived_minifed_path = '/%s/%s/archive/%s/%s'%(subfolder, self.slug, self.version, minified_name)
                self.archive_file(archived_source_path, source)
                self.archive_file(archived_minifed_path, minified_source)

            self.needs_render = False
            self.save()

            # else:
            #     print 'success but not different'
        else:
            # print 'error, dont save'
            self.error_source_content = error_message

        self.save()

    def get_file_name(self, minified=False):
        if minified:
            return u"%s.min%s"%(self.slug, self.get_extension())
        else:
            return u"%s%s"%(self.slug, self.get_extension())

    def get_archived_file_url(self, version, minified=False):

        subfolder = self.__class__.__name__.lower()
        source_name = self.get_file_name(minified)
        source_path = '/%s/%s/archive/%s/%s'%(subfolder, self.slug, version, source_name)
        url = "http://%s.s3.amazonaws.com%s" % (settings.AWS_STORAGE_BUCKET_NAME, source_path)
        return url

    def get_url(self, minified):
        if self.peg_revision==None or settings.DEBUG:
            if minified:
                if self.file_minified:
                    return '%s?v=%s'%(self.file_minified.url, self.version)
                else:
                    return None
            else:
                if self.file_source:
                    return '%s?v=%s'%(self.file_source.url, self.version)
                else:
                    return None
        else:
            return self.get_archived_file_url(self.peg_revision, minified)

    def archive_file(self, source_name, source_file_contents):
        
        conn = boto.connect_s3(settings.AWS_ACCESS_KEY_ID, settings.AWS_SECRET_ACCESS_KEY)
        bucket = conn.get_bucket(settings.AWS_STORAGE_BUCKET_NAME_MEDIA)
        
        headers={'Content-Type': self.get_header_type(), 'Cache-Control': 'max-age=%s'%(settings.CACHE_DURATION)}
        
        try:
            is_gzipped = getattr(settings, 'AWS_IS_GZIPPED', False)
            if is_gzipped:
                new_source_file_contents = self.compress_string(source_file_contents)
                headers['Content-Encoding'] = 'gzip'
                new_source_file_contents = source_file_contents
        except:
            pass


        k = Key(bucket)
        k.key = source_name
        k.set_contents_from_string(source_file_contents, headers=headers)
        k.set_acl('public-read')

    def compress_string(self, s):
        """Gzip a given string."""
        zbuf = StringIO()
        zfile = GzipFile(mode='wb', compresslevel=6, fileobj=zbuf)
        zfile.write(s)
        zfile.close()
        return zbuf.getvalue()




    def increment_version_number(self):
        #Ignore default increment
        pass

    def minify(self, source):
        #Override in subclass
        return source

    def get_source(self):
        children = self.get_children()


        #prepare all children by storing contents locally
        for child in children:
            # print '\n\n'
            child.store_raw_file()

        #return concatenated source files
        source = ''
        all_success = True
        all_errors = ''
        for child in children:
            # print '\n\n'
            rendered, success, error = child.render()
            if not success:
                all_success = False
                all_errors += "Error in item %s:\n%s\n\n"%(child, error)
                
            
            #Convert rendered content to unicode UTF-8
            try:
                rendered_unicode = unicode(rendered, "utf-8")                
            except:
                #conversion wont work if its already unicode
                rendered_unicode = rendered
            
            slug_decoded = child.slug.decode("utf-8")
            rendered_decoded = rendered_unicode
            source += u'\n\n\n/* --- SOURCE: %s --- */\n\n%s'%(slug_decoded, rendered_decoded)


        # unicode_src = source.decode('ascii')
        # utf8_src = unicode_src.encode('utf-8')

        return (source, all_success, all_errors)

    class Meta:
        abstract = True

    # def save(self, *args, **kwargs):
    #     super(BaseFrontendPackage, self).save(*args, **kwargs)
    #     print "ERROR? %s"%(self.error_source_content)


        


class CSSPackage(BaseFrontendPackage):
    class Meta:
        abstract = True

    def get_extension(self):
        return '.css'

    def get_header_type(self):
        return 'text/css'

    def minify(self, source):
        #Override in subclass
        return compress(source)


    # def get_children(self):
    #     return CSSResource.objects.filter(parent=self).order_by('order')



class JSPackage(BaseFrontendPackage):
    class Meta:
        abstract = True

    def get_extension(self):
        return '.js'

    def get_header_type(self):
        return 'application/javascript'

    def minify(self, source):
        #Override in subclass
        return minify(source, mangle=True, mangle_toplevel=True)

    # def get_children(self):
    #     return JSResource.objects.filter(parent=self).order_by('order')
    

class BaseFrontendResource(VersionableAtom, TitleAtom, OrderedItemAtom):

    help = {
        'custom_source': "Set the source of this resource",
        'file_source_url':"URL to an existing resource",
        'file_source_path':"Relative source path, if file_source_url is a package",
        'title':"",
        'slug':"The filename for this item, without '.min' or the file extension"
    }

    custom_source = models.TextField(_('custom source'), 
        help_text=help['custom_source'], null=True, blank=True)
    file_source_url = models.CharField(_('File Source'), max_length=255, 
        null=True, blank=True,
        help_text=help['file_source_url'])  
    file_source_path = models.CharField(_('File Source Path'), max_length=255, 
        null=True, blank=True, help_text=help['file_source_path']) 

    def generate_slug(self):
        return slugify(self.title)

    def edit_parent(self):
        style="style='width:278px;display:block;'"
        if self.parent:            
            try:
                object_type = type(self.parent).__name__
                url = reverse('admin:%s_%s_change' %(self.parent._meta.app_label,  self.parent._meta.model_name),  args=[self.parent.id] )
                return '<a href="%s" %s>&lt; Edit Parent</a>'%(url, style)
            except:
                return '<span %s>&nbsp;</span>'%(style)
        return '<span %s>&nbsp;</span>'%(style)

    def render(self):
        source, success, error_message = self.get_source()
        if not success:
            return (None, success, error_message) 
               
        compiled, success, error = self.compile(source)
        return (compiled, success, error)
    

    def compile(self, source):
        #OVERRIDE IN SUBCLASS
        success = True
        error = None
        return (source, success, error)
        

    def get_source(self):
        if self.custom_source:
            return (self.custom_source, True, None)
        elif self.file_source_url:
            
            if self.is_source_package():
                # print '%s is a source pagage, so download the files'%(self.file_source_url)
                get_package_contents(self.file_source_url, self.get_package_download_folder())

                if self.is_source_file():
                    # print '%s is a source file, so get contents from file...'%(self.get_package_original_source_folder())
                    file_source, success, error_message = get_file_contents(self.get_package_original_source_folder())
                    
                    if success:
                        return (file_source, True, None)
                    else:
                        return ('', success, error_message)

                else:

                    success, error = copy_directory(self.get_package_original_source_folder(), self.get_package_source_folder())
                    if success:
                        return ('', True, None)
                    else:
                        return (None, success, error)


                

            else:
                response = urllib2.urlopen(self.file_source_url)
                http_source = response.read()
                return (http_source, True, None)

        return ('', True, None)

    def store_raw_file(self):
        #Retreives any remote content or db content and renders it into files
        
        raw_source, success, error_message = self.get_source()
        if success:
            

            skip_file_write = self.is_source_package() == True and self.is_source_file() == False
            if not skip_file_write:
                # print 'store_raw_file: %s to %s/%s'%(self.slug, self.get_resource_temp_folder(), raw_source_name)
                write_to_file(self.get_resource_temp_name(), self.get_resource_temp_folder(), raw_source)    

    def delete_downloaded_files(self):
        if self.file_source_url != None:
            if os.path.exists(self.get_package_download_folder()):
                shutil.rmtree(self.get_package_download_folder())

            if os.path.exists(self.get_package_source_folder()):
                shutil.rmtree(self.get_package_source_folder())

    def get_extension(self):
        #Override in subclass
        return '.txt'

    def is_partial(self):
        return self.slug.startswith('_')

    def is_source_package(self):
        if self.file_source_url:
            is_zip = '.zip' in self.file_source_url
            is_gzip = 'tag.gz' in self.file_source_url
            if is_zip or is_gzip:
                return True
        return False

    def is_source_file(self):
        if self.file_source_path:
            #If there is a valid filename in the 
            return len(os.path.basename(self.file_source_path).split('.')) > 1
        return False

    def get_parent_temp_folder(self):
        subfolder = '%s_%s'%((self.__class__.__name__).lower(), slugify(settings.SITE_TITLE.lower()))
        directory = "/tmp/%s/%s"%(subfolder, self.parent.slug)   
        return directory 

    def get_resource_temp_name(self):
        return "%s.%s"%(self.slug, self.get_extension())

    def get_resource_temp_folder(self):
        subfolder = '%s_%s/%s'%((self.__class__.__name__).lower(), slugify(settings.SITE_TITLE.lower()), self.parent.slug)
        directory = "/tmp/%s"%(subfolder)   
        return directory 

    def get_package_download_folder(self):
        directory = "%s/downloaded/%s"%(self.get_parent_temp_folder(), self.slug)   
        return directory

    def get_package_original_source_folder(self):
        directory = "%s/%s"%(self.get_package_download_folder(), self.file_source_path)   
        return directory

    def get_package_source_folder(self):
        directory = "%s/%s"%(self.get_resource_temp_folder(), self.slug)   
        return directory        
    
    def get_src_directories(self):
        dirs = [self.get_resource_temp_folder()]
        if self.parent:
            return  dirs + self.parent.get_src_directories()
        else:
            return dirs


    

    def save(self, *args, **kwargs):
        
        source_has_changed = False
        download_path_has_changed = False
        if self.pk is not None:
            original = self.__class__.objects.get(pk=self.pk)
        else:
            original = None

        if original:


            original_source, original_success, original_source_error_message = original.get_source()
            current_source, original_success, current_source_error_message = self.get_source()
            if original_source != current_source:
                source_has_changed = True


            if self.file_source_url != original.file_source_url or self.file_source_path != original.file_source_path:
                download_path_has_changed = True

            if download_path_has_changed:
                self.delete_downloaded_files()


        super(BaseFrontendResource, self).save(*args, **kwargs)


        if (source_has_changed or download_path_has_changed) and self.parent:
            #Re-save parent
            self.parent.request_render()

    @staticmethod
    def autocomplete_search_fields():
        return ("admin_note__icontains","title__icontains",)

    def __unicode__(self):
        return '%s - %s'%(self.title, self.slug)

    class Meta:
        ordering = ['order']
        abstract = True

class CSSResource(BaseFrontendResource):
    COMPILER_CSS = 'css'
    COMPILER_SCSS = 'scss'
    COMPILER_SASS = 'sass'
    # COMPILER_LESS = 'less'
    COMPILER_CHOICES = (
        (COMPILER_CSS, "CSS (None)"),
        (COMPILER_SCSS, "SASS")
    )
    #TODO:
    # clevercss
    # less_ruby
    # stylus

    compiler = models.CharField("Compiler / Preprocessor", max_length=255, 
        choices=COMPILER_CHOICES, default=COMPILER_CSS )


    parent = models.ForeignKey('core.CSSPackage')

    def get_extension(self):
        return self.compiler

    def compile(self, source):
        success = True
        error = None
        if self.compiler == CSSResource.COMPILER_CSS:
            return (source, success, error)

        if self.compiler == CSSResource.COMPILER_SCSS or \
            self.compiler == CSSResource.COMPILER_SASS:

            if self.is_partial() or self.is_source_package():
                return ('', success, error)
            else:
            
                try:
                    # Insert comments around import statements for easier debugging.
                    import_comments_string = source.replace("'", '"')
                    
                    def insert_comments(matchobj):
                      filenames = re.findall(r'"(.*?)"', matchobj.group(0))
                      filename = ' '.join(filenames)
                      return '\n\n/* --- IMPORT: %s --- */\n%s'%(filename, matchobj.group(0))

                    import_comments_string = re.sub(r'@import "(.*?)"', insert_comments, import_comments_string)

                    compiled = sass.compile(string=import_comments_string,include_paths=self.get_src_directories())
                    return (compiled, success, error)

                except Exception, err:
                    error = 'Error compiling %s: %s - %s'%(self.title, traceback.format_exc(), sys.exc_info()[0])
                    print error
                    success = False
                    return (error, success, error)
                    

        #TODO -- add other compilation versions

        return (source, success, error)



    class Meta:
        abstract = True

class JSResource(BaseFrontendResource):
    COMPILER_JS = 'js'
    COMPILER_COFFEE = 'coffee'

    #TODO:
    # rjsmin
    # yui_js
    # closure_js

    COMPILER_CHOICES = (
        (COMPILER_JS, "JS (None)"),
        (COMPILER_COFFEE, "Coffeescript"),
    )
    compiler = models.CharField("Compiler / Preprocessor", max_length=255, 
        choices=COMPILER_CHOICES, default=COMPILER_JS )

    parent = models.ForeignKey('core.JSPackage')

    def get_extension(self):
        return self.compiler

    def compile(self, source):
        success = True
        error = None
        if self.compiler == JSResource.COMPILER_JS:
            return (source, success, error)

        #TODO -- add other compilation versions

        return (source, success, error)

    class Meta:
        abstract = True


def write_to_file(filename, directory, content):
    if not os.path.exists(directory):
        os.makedirs(directory)

    file_path = "%s/%s"%(directory, filename)
    if not os.path.exists(file_path):
        open(file_path, 'w').close()

    # encoded = content.encode('utf-8', ignore)
    try:
        f = codecs.open(file_path, 'w+',encoding='utf-8')
        f.write(content)
    except Exception, err:
        error_message = 'Error storing file %s: %s - %s'%(file_path, traceback.format_exc(), sys.exc_info()[0])
        print error_message

def get_file_contents(filename):
    if not os.path.exists(filename):
        return (None, False, "Error: File %s doesn't exist"%(filename))
     
    try:
        with open(filename, 'r') as content_file:
            content = content_file.read()       
            return (content, True, None)
    except Exception, err:
        error_message = 'Error getting file contents %s: %s - %s'%(filename, traceback.format_exc(), sys.exc_info()[0])
        return (None, False, error_message)

def get_package_contents(url, target_dir):
    # print 'get unzipped: %s'%(url)
    if not os.path.exists(target_dir):
        # print 'create package directory: %s'%(target_dir)
        os.makedirs(target_dir)

    temp_file_name = os.path.basename(url)
    name = os.path.join(target_dir, temp_file_name)
    
    
    if not os.path.exists(name):
        # print 'download file to %s'%(name)
        try:
            name, hdrs = urllib.urlretrieve(url, name)
        except IOError, e:
            # print "Can't retrieve %r to %r: %s" % (url, target_dir, e)
            return
    else:
        # print 'file exists. its all good.'
        pass

    is_zip = '.zip' in url
    is_gzip = 'tag.gz' in url

    if is_zip:

        try:
            zf = zipfile.ZipFile(name)
        except zipfile.error, e:
            print "Bad zipfile (from %r): %s" % (url, e)
            return
        zf.extractall(target_dir)



    elif is_gzip:
        print "TODO -- add gzip support"

def copy_directory(src, dest):
    success = True
    error = None
    # print 'copy from %s to %s'%(src, dest)
    if os.path.exists(dest):
        try:
            shutil.rmtree(dest)
        except shutil.Error as e:
            success = False
            error = 'Directory not removed. Error: %s' % e

    try:
        shutil.copytree(src, dest)
    # Directories are the same
    except shutil.Error as e:
        success = False
        error = 'Directory not copied. Error: %s' % e

    # Any error saying that the directory doesn't exist
    except OSError as e:
        success = False
        error = 'Directory not copied. Error: %s' % e

    return (success, error)





from django.contrib import admin
from django.contrib import messages
from django.utils.safestring import mark_safe

from carbon.atoms.admin.content import *
from carbon.atoms.admin.taxonomy import *

from reversion.admin import VersionAdmin
from django_inline_wrestler.admin import TabularInlineOrderable

from carbon.atoms.admin.content import BaseVersionableAdmin, BaseVersionableTitleAdmin

# from .models import *
from .forms import *



class TemplateAdmin(VersionAdmin, BaseVersionableAdmin):
    
    form = TemplateAdminForm
    prepopulated_fields = {"slug": ("title",)}

    core_fields = (
        ('title','slug'),
        'file_template',
        'custom_template',
    )
    meta_fields = BaseVersionableAdmin.meta_fields
    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )    

    search_fields = ('title', 'slug', 'admin_note', 'custom_template')
    list_display = ('title', 'slug')


class BaseFrontendPackageAdmin(VersionAdmin, BaseVersionableTitleAdmin):

    def archived_versions(self, obj):
        url = obj.get_archived_file_url(obj.version, True)
        return mark_safe("You can download archived versions using this URL schema:<br /><a href='%s' target='_blank'>%s</a>"%(url, url))
    

    def rerender(modeladmin, request, queryset):
        for object in queryset:
            object.render(True, True)
            if object.error_source_content == None:
                messages.add_message(request, messages.SUCCESS, 'Package %s was re-rendered'%(object.title))
            else:
                messages.add_message(request, messages.ERROR, 'An error was encountered when rendering %s: %s'%(object.title, object.error_source_content))

    rerender.short_description = "Re-Render Package"

    prepopulated_fields = {"slug": ("title",)}
    core_fields = (
        ('title','slug'),
        # ('source','minified'),
        ('file_source', 'file_minified'),
    )
    info_fields = (
        ('version','peg_revision',),
        'archived_versions',
        'error_source_content',
        'needs_render'
    )
    meta_fields = BaseVersionableTitleAdmin.meta_fields
    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Info", {
            'fields': info_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    ) 
    #readonly_fields = BaseVersionableTitleAdmin.readonly_fields + ('file_source','file_minified')

    readonly_fields = (
        "version", "created_date", "created_by", "modified_date", "modified_by",
        "error_source_content", "archived_versions"
    ) 

    actions = [rerender]


    def response_change(self, request, object):
        response = super(BaseFrontendPackageAdmin, self).response_change(request, object)
        object.render()

        if object.error_source_content != None:
            messages.add_message(request, messages.ERROR, 'An error was encountered when rendering %s: %s'%(object.title, object.error_source_content))

        return response

    def response_add(self, request, object):
        response = super(BaseFrontendPackageAdmin, self).response_add(request, object)
        object.render()

        if object.error_source_content != None:
            messages.add_message(request, messages.ERROR, 'An error was encountered when rendering %s: %s'%(object.title, object.error_source_content))

        return response



class CSSPackageAdmin(BaseFrontendPackageAdmin):
    pass

class JSPackageAdmin(BaseFrontendPackageAdmin):
    pass    



class BaseFrontendResourceAdmin(VersionAdmin, BaseVersionableTitleAdmin):

    prepopulated_fields = {"slug": ("title",)}
    core_fields = (
        ('edit_parent',),
        ('title','slug'),
        ('compiler',),
        ('file_source_url'),        
    )
    source_fields = (
        'custom_source',
    )
    meta_fields = BaseVersionableTitleAdmin.meta_fields
    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Source", {
            'fields': source_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    ) 
    readonly_fields = BaseVersionableTitleAdmin.readonly_fields + ('edit_parent',)


    

class CSSResourceAdmin(BaseFrontendResourceAdmin):
    form = CSSResourceAdminForm

class JSResourceAdmin(BaseFrontendResourceAdmin):
    form = JSResourceAdminForm

class BaseFrontendResourceInline(admin.StackedInline):

    def edit_item(self, obj):
        return mark_safe("<a href='%s' target='_blank'>See Item Details and Revision History></a>"%(obj.edit_item_url))
    
    #fields = ('order','title', 'compiler','file_source_url','edit_item',)
    readonly_fields = BaseVersionableTitleAdmin.readonly_fields + ('edit_item',)
    ordering = ("order",)
    sortable_field_name = 'order'
    core_fields = (
        ('title','slug', 'order', 'edit_item'),
        ('compiler', 'file_source_url', 'file_source_path'),
    )
    source_fields = (
        'custom_source',
    )
    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Source", {
            'fields': source_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
    ) 
    classes = ('grp-collapse grp-open',)
    prepopulated_fields = {"slug": ("title",)}
    
    extra = 0

class CSSResourceInline(BaseFrontendResourceInline):
    form = CSSResourceAdminForm
    fk_name = 'parent'
    prepopulated_fields = {"slug": ("title",)}

class JSResourceInline(BaseFrontendResourceInline):
    form = JSResourceAdminForm
    fk_name = 'parent'
    prepopulated_fields = {"slug": ("title",)}

    core_fields = (
        ('title','slug', 'order', 'edit_item'),
        ('file_source_url', 'file_source_path'),
    )
    source_fields = (
        'custom_source',
    )
    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Source", {
            'fields': source_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
    ) 




class MenuItemInline(TabularInlineOrderable):
    #model = MenuItem  
    form = MenuItemForm  
    fk_name = 'parent'
    
    autocomplete_lookup_fields = {
        'generic': [['content_type', 'object_id']],
        'fk': [],
    }
    fields = ('order','title','content_type', 'object_id','path_override','edit_item',)
    readonly_fields = BaseVersionableAdmin.readonly_fields + ('path','edit_item',)
    extra = 0


class MenuItemAdmin(BaseVersionableAdmin):
            
    autocomplete_lookup_fields = {
        'generic': [['content_type', 'object_id']],
        'fk': ('parent',),
    }
    raw_id_fields = ( 'parent',)

    core_fields = (
        ('edit_parent','parent'),
        ('title','slug'),
        ('content_type', 'object_id',),
        ('path_override',),
        ('path',),
        ('publication_status','css_classes',),
        ('target','extra_attributes')
    )
    prepopulated_fields = {"slug": ("title",)}
    ordering = ("hierarchy",)

    
    list_display = ( "admin_hierarchy", "title", 'path', 'publication_status')
    list_display_links = ('title',)
    fieldsets = (
        ("Main", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Meta", {
            'fields': BaseVersionableAdmin.meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )
    readonly_fields = BaseVersionableAdmin.readonly_fields + ('path','edit_parent')
    # inlines = [MenuItemInline]

class AdminAppLinkInline(TabularInlineOrderable):

    # model = AdminLinkItem
    fk_name = 'parent'
    ordering = ("order",)
    extra = 0
    fieldsets = (
        ( 'Links', { 'fields': ( 'order', 'model_path', ) } ),
    )

class AdminLinkInline(TabularInlineOrderable):

    # model = AdminLink
    fk_name = 'parent'
    ordering = ("order",)
    extra = 0
    fieldsets = (
        ( 'Links', { 'fields': ( 'order', 'title', 'url') } ),
    )    
  
class AdminAppGroupAdmin(VersionAdmin, BaseVersionableAdmin):
    #inlines = [AdminAppLinkInline]
    core_fields = (
        ('title', 'open_by_default'),
    )
    meta_fields = BaseVersionableTitleAdmin.meta_fields
    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    ) 

    list_display = ('title', 'order',)
    list_editable = ('order',)

class AdminSidebarAdmin(VersionAdmin, BaseVersionableAdmin):
    #inlines = [AdminLinkInline]
    core_fields = (
        ('title',),
    )
    meta_fields = BaseVersionableTitleAdmin.meta_fields
    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    ) 

    list_display = ('title', 'order',)
    list_editable = ('order',)

class LegacyURLRefererInline(admin.TabularInline):
    #model = LegacyURLReferer

    fields = ('referer_title','referer_url', 'created_date')
    extra = 0

    readonly_fields = BaseVersionableAdmin.readonly_fields


class LegacyURLAdmin(BaseVersionableAdmin):

    def visit_old_link(self, obj):
        return mark_safe("<a href='%s%s' target='_blank'>Visit Old Link</a>"%(settings.LEGACY_URL_ARCHIVE_DOMAIN, obj.url))
    

    def test_redirect(self, obj):
        return mark_safe("<a href='%s' target='_blank'>Test Redirect</a>"%(obj.url))
    
    
    core_fields = (
        ('url'),
        ( 'content_type', 'object_id',),
        ('path_override'),
        ('path'),
        ('visit_old_link','test_redirect')
    )
    

    list_display = ( "url", "path",'created_date', )
    autocomplete_lookup_fields = {
        'generic': [['content_type', 'object_id']],
        'fk': [],
    }

    fieldsets = (
        ("Main", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Meta", {
            'fields': BaseVersionableAdmin.meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )
    readonly_fields = BaseVersionableAdmin.readonly_fields + ('path','visit_old_link','test_redirect')

    # inlines = [LegacyURLRefererInline]    

# admin.site.register(Template, TemplateAdmin)
# admin.site.register(MenuItem, MenuItemAdmin)
# admin.site.register(LegacyURL, LegacyURLAdmin)



from os.path import join, split

from django import forms
from django.conf import settings
from django.core.files.storage import FileSystemStorage, DefaultStorage
from django.core.files.uploadedfile import InMemoryUploadedFile, TemporaryUploadedFile
from django.utils.html import escape
from django.utils.module_loading import import_string


from ckeditorfiles.widgets import CKEditorInlineWidget
from ckeditorfiles.widgets import CKEditorWidget

from carbon.atoms.models.media import BaseSecureAtom
from .models import *

class FormFieldInlineAdminForm(forms.ModelForm):
    content = forms.CharField(widget=CKEditorInlineWidget(config=settings.CKEDITOR_CONFIGS['page_content_ckeditor']), required=False)
    
    # IMPLEMENT
    # class Meta:
    #     model = FormField
    #     fields = '__all__'


class FormAdminForm(forms.ModelForm):
    content = forms.CharField(widget=CKEditorWidget(config=settings.CKEDITOR_CONFIGS['page_content_ckeditor']), required=False)
    submission_content = forms.CharField(widget=CKEditorWidget(config=settings.CKEDITOR_CONFIGS['page_content_ckeditor']), required=False)
    
    class Meta:
        # model = Form
        fields = '__all__'


class FormEntryForm(forms.ModelForm):
    
    form_field_prefix = 'form_field_'
    form_fields = []
    auto_populate_get_values = True #Auto-populate values from GET into the forms initial values

    # IMPLEMENT
    # field_model = FieldEntry
    # class Meta:
    #     model = FormEntry
    #     fields = ['form']

    def __init__(self, form_schema, request, *args, **kwargs):
        self.form_schema = form_schema
        self.request = request
        super(FormEntryForm, self).__init__(*args, **kwargs)

        self.fields['form_schema'].initial = self.form_schema

        if self.form_schema:
            self.model_form_fields = self.form_schema.get_all_fields()
            
            for model_field in self.model_form_fields:
                form_field = model_field.get_form_field()
                key = self.form_field_prefix+model_field.slug
                self.fields[key] = form_field
                self.form_fields.append(form_field)


            #Auto-populate form fields with get values if applicable
            is_get = self.request.method=="GET"
            if is_get and self.auto_populate_get_values:
                for key in self.request.GET:
                    has_corresponding_form_field = key in self.fields
                    if has_corresponding_form_field:
                        self.fields[key].initial = self.request.GET[key]

        
        

    

    def get_form_fields(self):
        return self.form_fields

    def cleaned_value(self, value):
        value = escape(value)
        return value

    def _clean_fields(self):
        
        super(FormEntryForm, self)._clean_fields()

        for field_key in self.fields:
            if self.form_field_prefix in field_key:
                field = self.fields[field_key]
                raw_value = self._raw_value(field_key)
            
                try:
                    field.widget.model_field.validate(raw_value, self)
                except ValidationError as e:
                    self.add_error(field_key, e)



    def save(self, **kwargs):
        entry = super(FormEntryForm, self).save()
        
        # entry.record_entry(self)
        for field_key in self.fields:
            if self.form_field_prefix in field_key:
                field = self.fields[field_key]
                raw_value = self._raw_value(field_key)
                value = self.cleaned_value(raw_value)
                model_field = field.widget.model_field

                #TODO-- add secure file handling...
                if value and field.widget.model_field.is_multipart:

                    type = raw_value.__class__.__name__
                    # print 'TYPE? %s'%(type)
                    if isinstance(raw_value, InMemoryUploadedFile) or isinstance(raw_value, TemporaryUploadedFile):
                        
                        
                        file_upload_path = join('form_uploads', str(self.form_schema.slug), str(entry.pk), str(model_field.slug), raw_value.name)
                            # if settings.DEBUG:
                            #     print 'FILE UPLOAD PATH: %s'%(file_upload_path)
                        try:
                            
                            if model_field.type == FormField.SECURE_FILE:
                                
                                secure_file_storage = import_by_path(settings.SECURE_MEDIA_STORAGE)()
                                value = secure_file_storage.save(file_upload_path, raw_value)

                                key_name = "%s/%s"%(settings.AWS_MEDIA_FOLDER, value)
                                BaseSecureAtom.make_private(settings.AWS_STORAGE_BUCKET_NAME_MEDIA_SECURE, key_name)

                            else:
                                
                                file_storage = DefaultStorage()
                                value = file_storage.save(file_upload_path, raw_value)
                        except:
                            print "Error uploading file to %s/%s"%(file_upload_path, raw_value)
                            value = file_upload_path
                
                
                field_entry, created = self.field_model.objects.get_or_create(form_entry=entry, form_field=field.widget.model_field)
                # if created:
                #     print 'created new field entry: %s, %s, %s'%(field_entry, entry.pk, field.widget.model_field.title)
                # else:
                #     print 'Update field entry %s'%(value)
                field_entry.value = field_entry.get_compressed_value(value)
                field_entry.save()
        

        return entry


from django.views.generic import DetailView, ListView
from django.conf import settings
from django.views.generic.edit import FormMixin, ProcessFormView
try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model


from carbon.atoms.views.abstract import *
from carbon.atoms.views.content import *

from .models import *
from .forms import *

class BlogCommentMixin(FormMixin, ProcessFormView):

    form_class = BlogCommentForm
    was_posted = False


    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        self.was_posted = False
        if self.object.allow_comments:
            form = self.get_form()        
        else:
            form = None

        context = self.get_context_data(object=self.object,form=form)
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        self.was_posted = True
        
        if self.object.allow_comments:
            form = self.get_form()
            if form.is_valid():
                return self.form_valid(form)
            else:
                return self.form_invalid(form)
        else:
            return self.get(*args, **kwargs)

    # PUT is a valid HTTP verb for creating (with a known URL) or editing an
    # object, note that browsers only support POST for now.
    def put(self, *args, **kwargs):
        return self.post(*args, **kwargs)


    def get_form_kwargs( self ):
        kwargs = super( BlogCommentMixin, self ).get_form_kwargs()
        kwargs['request'] = self.request
        kwargs['was_posted'] = self.was_posted
        return kwargs

    def get_success_url(self):
        return self.object.get_absolute_url()

    def form_valid(self, form):
        form.save()
        return super(BlogCommentMixin, self).form_valid(form)

class BaseBlogDetailView(BlogCommentMixin):

    def get_context_data(self, **kwargs):
        context = super(BaseBlogDetailView, self).get_context_data(**kwargs)
            
        # print 'todo: Get next/previous, and current tag'
        
        context['next'] = None
        context['previous'] = None

        return context

class BaseBlogListView(DetailView):

    def get_context_data(self, **kwargs):
        context = super(BaseBlogListView, self).get_context_data(**kwargs)
            
        tag_model = get_model(settings.BLOG_TAG_MODEL.split('.')[0], settings.BLOG_TAG_MODEL.split('.')[1])
        category_model = get_model(settings.BLOG_CATEGORY_MODEL.split('.')[0], settings.BLOG_CATEGORY_MODEL.split('.')[1])
        #article_model = get_model(settings.BLOG_ARTICLE_MODEL.split('.')[0], settings.BLOG_ARTICLE_MODEL.split('.')[1])
        
        context['tags'] = tag_model.objects.published()
        context['categories'] = category_model.objects.published()

        return context


class BlogArticleDetailView(NonAdminCachableView, PublishableView, AddressibleView, BaseBlogDetailView):


    # model = BlogArticle
    def get_object_query(self, queryset, path):
        return queryset.filter(path=path).select_related('template', 'category').prefetch_related('tags', 'related').get()

    
class BlogTagView(NonAdminCachableView, PublishableView, AddressibleView, HasChildrenView, BaseBlogListView):

    # model = BlogTag
    pass

class BlogCategoryView(NonAdminCachableView, PublishableView, AddressibleView, HasChildrenView, BaseBlogListView):

    # model = BlogCategory
    pass

class BlogListView(NonAdminCachableView, PublishableView, AddressibleView, HasChildrenView, BaseBlogListView):

    # model = Page

    # def get_children(self):
    # articles = BlogArticle.objects.published()
    # return [article for article in articles if article.is_published()]
    pass
    

class BlogTagListView(NonAdminCachableView, PublishableView, AddressibleView, HasChildrenView, BaseBlogListView):

    # model = Page
    # def get_children(self):
    # articles = BlogTag.objects.published()
    # return [article for article in articles if article.is_published()]
    pass     

class BlogContributorListView(NonAdminCachableView, PublishableView, AddressibleView, HasChildrenView, BaseBlogListView):

    # model = settings.BLOG_ROLE_USER_MODEL
    # def get_children(self):
    # articles = BlogTag.objects.published()
    # return [article for article in articles if article.is_published()]
    pass  

class BlogContributorDetailView(NonAdminCachableView, PublishableView, AddressibleView, DetailView):

    # model = settings.BLOG_ROLE_USER_MODEL
    # def get_children(self):
    # articles = BlogTag.objects.published()
    # return [article for article in articles if article.is_published()]
    pass  






import csv

from django.http import HttpResponse

def get_repr(value): 
    if callable(value):
        return '%s' % value()
    return value

def get_field(instance, field):
    field_path = field.split('.')
    attr = instance
    for elem in field_path:
        try:
            attr = getattr(attr, elem)
        except AttributeError:
            return None
    return attr


def ModelToCSV(request, queryset, fields, filename="output.csv"):
	MANY_RELATED_MANAGER = "ManyRelatedManager"

	capitalized_fields = [w.replace("_", " ").replace("-", " ").replace("  ", " ").title() for w in fields]
	csv_data = []
	csv_data.append(capitalized_fields)

	
	# -- Rows
	for item in queryset:
		row = []
		for field in fields:

			if '__' in field:

				pieces = field.split('__')
				current_item = item


				for piece in pieces:
					if current_item:
						current_item = getattr(current_item, piece)

				row.append(unicode(current_item).encode("ascii", "ignore"))

			else:

				try:
					attr = get_repr(get_field(item, field))
					field_type = attr.__class__.__name__
					#row.append(unicode("CN: "+field_type).encode("utf-8"))			
					print "field_type: %s %s"%(field_type, field)

					if field_type == MANY_RELATED_MANAGER:
						ms = []
						for m in attr.all():
							ms.append(unicode(m).encode('ascii', 'ignore'))

						row.append(u'; '.join(ms).encode('ascii', 'ignore'))
					elif field_type == 'property':

						row.append(unicode(attr).encode("ascii", "ignore"))

					elif field_type == 'instancemethod':
						
						attr = getattr(item, field)()

						row.append(unicode(attr).encode("ascii", "ignore"))

					else:

						row.append(unicode(attr).encode("ascii", "ignore"))
					
				except:
					row.append(u"Not found")

		csv_data.append(row)

	return CSVResponse(request, csv_data, filename)


def ArrayToCSV(request, data, filename="output.csv"):	

	csv_data = []
	
	# -- Rows
	for fields in data:
		row = []
		for field in fields:
			try:
				row.append(unicode(field).encode("ascii", "ignore"))
			except:
				row.append(u"Not found")

		csv_data.append(row)

	return CSVResponse(request, csv_data, filename)

def CSVResponse(request, data, filename="output.csv"):
	# Create the HttpResponse object with the appropriate CSV header.

	#response = HttpResponse(content_type="text/plain")

	response = HttpResponse(content_type="text/csv")
	response['Content-Disposition'] = 'attachment; filename="'+filename+'"'
   

	writer = csv.writer(response)

	for line in data:
		writer.writerow(line)

	return response



# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from django.conf import settings


# https://pypi.python.org/pypi/django-donottrack/0.1
def donottrack(request):
    """
    Adds ``donottrack`` to the context, which is ``True`` if the ``HTTP_DNT``
    header is ``'1'``, ``False`` otherwise.

    This context processor requires installtion of the
    ``donottrack.middleware.DoNotTrackMiddleware``.

    Note that use of this context processor is not strictly necessary. (Though
    it is quite convenient.) If you are using the
    ``django.core.context_processors.request`` context processor, you have
    access to ``{{ request.donottrack }}``.

    """
    # We could just use the ``donottrack.utils.get_donottrack`` function rather
    # than rely on the middlware, but we want to require the middleware so that
    # the VARY header is properly patched to account for DNT.
    try:
        return {'donottrack': request.donottrack}
    except AttributeError:
        raise AttributeError("'WSGIRequest' object has no attribute 'donottrack'"
            " - 'donottrack.middleware.DoNotTrackMiddleware' must be in your"
            " MIDDLEWARE_CLASSES")

def site(request):

    try:
        return {
            'site': request.site
        }
    except:
        return {
            'site': None
        }
    

def custom_settings(request):
    object = {}

    try:

        all_settings = settings.CUSTOM_SETTINGS 
        for setting in all_settings:
            if hasattr(request, setting):
                value = getattr(request, setting)
                object[setting] = value         
    except:
        pass

    return object


def impersonating(request):
    object = {}

    return {
            'impersonating': request.impersonating
        }



from django.contrib import admin
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe

from carbon.atoms.models.content import PublishableAtom




class BaseVersionableAdmin(admin.ModelAdmin):

    class Media:        
        css = {
            "all": ('admin/css/adjustments.css',)
        }

    readonly_fields = (
        "version", "created_date", "created_by", "modified_date", "modified_by",
    )   

    
    meta_fields = (
        ('version'),
        ('created_date', 'created_by'),
        ('modified_date', 'modified_by'),
        'admin_note'
    )

    fieldsets = (
        
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )

    autocomplete_lookup_fields = {
        'fk': (),
        'm2m': ()
    }
    raw_id_fields = ( 'created_by', 'modified_by')

    list_filter = (
            "created_by", "modified_by", )

    search_fields = ('admin_note',)

    def save_model(self, request, obj, form, change):
        if not getattr(obj, "created_by"):
            obj.created_by = request.user
        obj.modified_by = request.user


        super(BaseVersionableAdmin, self). save_model(request, obj, form, change)



class BaseVersionableTitleAdmin(BaseVersionableAdmin):

    core_fields = (
        ('title','slug'),

    )
    meta_fields = BaseVersionableAdmin.meta_fields
    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )
    search_fields = ('title','admin_note')

class BaseContentAdmin(admin.ModelAdmin):
    class Media:
        css = {
            'all': ('admin/css/adjustments.css',)
        }
        js = ('animations.js', 'actions.js')

    def admin_hierarchy(self, obj):
        return mark_safe(obj.admin_hierarchy)
    
    
    autocomplete_lookup_fields = {
        'fk': ('image', 'published_by', 'template'),
        'm2m': ()
    }
    raw_id_fields = ( 'image', 'published_by', 'template')
    
    list_display = ( "admin_hierarchy", "path",  "title", "publication_status",)
    list_display_links = ( "admin_hierarchy", "path", "title",)
    list_filter = (
            "publication_status", "created_by", "modified_by", 
            'published_by','is_searchable','in_sitemap',
            'sitemap_changefreq','sitemap_priority','noindex','nofollow',
            'shareable','social_share_type')
    ordering = ("hierarchy",)

    search_fields = ('title', 'content', 'synopsis', 'admin_note')


    prepopulated_fields = {"slug": ("title",)}
    
    readonly_fields = (
        "version", "created_date", "created_by", "modified_date", "modified_by",
         "path", "path_generated", "uuid", 'image_preview', 'edit_template'
    )
    


    core_fields = (
        ('title','slug'),
        ('publication_status'),
        ('template', 'edit_template'),
        ('image','image_preview'),
        'synopsis',
        'content',
        
    )

    publication_fields = (
        ('publication_date', 'published_by'),
        ('publish_on_date', 'expire_on_date'),
        'facebook_author_id',
        'twitter_author_id',
        'google_author_id'
    )
   
    path_fields = (
        ('path', 'uuid'),
        ('path_generated', 'path_override'),
        ('temporary_redirect', 'permanent_redirect'),
        'order'

    )

    seo_fields = (
        'page_meta_description',
        'page_meta_keywords',
        ('is_searchable','in_sitemap'),
        ('sitemap_changefreq','sitemap_priority'),
        ('noindex','nofollow')
    )

    social_fields = (
        ('shareable','social_share_type'),
        'tiny_url',
        'social_share_image',        
    )
    
    meta_fields = BaseVersionableAdmin.meta_fields
    


    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Path", {
            'fields': path_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Publication", {
            'fields': publication_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        
        ("Search Engine Optimization", {
            'fields': seo_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Social Integration", {
            'fields': social_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )

    def save_model(self, request, obj, form, change):
       

        if obj.pk is not None and obj.publication_status == PublishableAtom.PUBLISHED:
            original = type(obj).objects.get(pk=obj.pk)
            if original.publication_status != obj.publication_status:
                if not obj.published_by:
                    obj.published_by = request.user

        super(BaseContentAdmin, self). save_model(request, obj, form, change)


class HierarchicalContentAdmin(BaseContentAdmin):
    

    
    autocomplete_lookup_fields = BaseContentAdmin.autocomplete_lookup_fields
    fk_fields_list = list(autocomplete_lookup_fields['fk'])
    fk_fields_list.insert(0, 'parent')
    autocomplete_lookup_fields['fk'] = tuple(fk_fields_list)


    raw_id_fields = BaseContentAdmin.raw_id_fields
    raw_id_fields_list = list(raw_id_fields)
    raw_id_fields_list.insert(0, 'parent')
    raw_id_fields = tuple(raw_id_fields_list)


    core_fields = BaseContentAdmin.core_fields
    core_fields_list = list(core_fields)
    core_fields_list.insert(0, ('edit_parent','parent'))
    core_fields = tuple(core_fields_list)

    list_filter = BaseContentAdmin.list_filter
    list_filter_list = list(list_filter)
    list_filter_list.insert(0, 'parent')
    list_filter = tuple(list_filter_list)

    path_fields = BaseContentAdmin.path_fields
    publication_fields = BaseContentAdmin.publication_fields
    seo_fields = BaseContentAdmin.seo_fields
    social_fields = BaseContentAdmin.social_fields
    meta_fields = BaseVersionableAdmin.meta_fields

    readonly_fields = BaseContentAdmin.readonly_fields + ('edit_parent',)

    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Path", {
            'fields': path_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Publication", {
            'fields': publication_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        
        ("Search Engine Optimization", {
            'fields': seo_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Social Integration", {
            'fields': social_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )



class BaseSlideInlineAdmin(admin.TabularInline):
    
    #model = SlideClass
    extra = 0

    autocomplete_lookup_fields = {
        'fk': ('slide_image',),
    }
    raw_id_fields = ( 'slide_image',)

    def preview(self, obj):
        if obj.slide_image:
            try:
                edit_url = obj.slide_image.edit_item_url
                return mark_safe("<img src='%s' alt='%s preview'/><br /><a href='%s' >Edit Image &gt;</a>"%(obj.slide_image.thumbnail.url, obj.slide_image.title, edit_url))
            except:
                return ""
        return ''


    readonly_fields = ('preview',)
    fields = (
        'order',
        'slide_image',
        'preview',
    )


from django.utils import timezone

from django.db import models
from django.conf import settings
from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse_lazy
from django.utils.functional import cached_property
from django.utils.html import strip_tags
from django.utils.text import Truncator
from django.utils.translation import ugettext_lazy as _
try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model

from carbon.atoms.models.abstract import VersionableAtom

from carbon.atoms.models.content import *

class BlogTag(TagMolecule):
    publish_by_default = True
    # default_template = 'blog_tag'
    # item_class = BlogArticle
    # url_domain = settings.BLOG_TAG_DOMAIN

    class Meta:
        abstract = True

    def get_absolute_url(self):
        if self.is_external:
            return self.path_override
        return reverse_lazy('blog_tag', kwargs = {'path': self.get_url_path() })   

class BlogCategory(CategoryMolecule):
    publish_by_default = True
    # default_template = 'blog_category'
    # item_class = BlogArticle
    # url_domain = settings.BLOG_CATEGORY_DOMAIN

    class Meta:
        verbose_name_plural = 'Blog categories'
        abstract = True

    def get_absolute_url(self):
        if self.is_external:
            return self.path_override
        return reverse_lazy('blog_category', kwargs = {'path': self.get_url_path() })   

class BlogArticleRole(VersionableAtom):
    # url_domain = settings.BLOG_ARTICLE_DOMAIN

    class Meta:
        abstract = True

    help = {
        'order': "Display order",
        'role': "Role that user plays in this article"
    }

    try:
        BLOG_ROLE_CHOICES = settings.BLOG_ROLE_CHOICES        
    except:
        AUTHOR = 'author'
        EDITOR = 'editor'
        PUBLISHER = 'publisher'
        BLOG_ROLE_CHOICES = (
            (AUTHOR, _("Author")),
            (EDITOR, _("Editor")),
            (PUBLISHER, _("Publisher")),
        )

    order = models.IntegerField(default=0, help_text=help['order'])   
    
    # YOU GOTTA IMPLEMENT THIS:
    article = models.ForeignKey('blog.BlogArticle')
    user = models.ForeignKey(settings.BLOG_ROLE_USER_MODEL)
    role = models.CharField(_('Role'), max_length=255, 
        choices=BLOG_ROLE_CHOICES, help_text=help['role'])


class BlogArticle(ContentMolecule):
    # default_template = 'blog_article'

    help = {
        'allow_comments': "Allow comments on article"
    }

    class Meta:
        abstract = True
   
    allow_comments = models.BooleanField(default=False, 
        help_text=help['allow_comments'])
  
    related = models.ManyToManyField('self', blank=True, symmetrical=True)
    tags = models.ManyToManyField('blog.BlogTag', blank=True)
    category = models.ForeignKey('blog.BlogCategory', blank=True, null=True, 
        on_delete=models.SET_NULL)

    def build_path(self, path_attribute='slug', parent_path_attribute='path'):

        if self.category:
            return "%s%s/" % (self.category.path, self.slug)
        else:
            return "/%s/" % self.slug

    def get_absolute_url(self):
        if self.is_external:
            return self.path_override
        return reverse_lazy('blog_article', kwargs = {'path': self.get_url_path() }) 
    


class BlogComment(UserInputMolecule):
    class Meta:
        abstract = True
        ordering = ['created_date']
   

    article = models.ForeignKey('blog.BlogArticle')


    in_response_to = models.ForeignKey('self', blank=True, null=True)

    is_deleted = models.BooleanField(default=False)

    @cached_property
    def level(self):
        if not self.in_response_to:
            return 1
        else:
            return self.in_response_to.level+1

    @cached_property
    def is_draft(self):
        return self.moderation_status == ModerationAtom.SUBMITTED

    @cached_property
    def is_published(self):
        return (self.moderation_status == ModerationAtom.PUBLISHED or self.moderation_status == ModerationAtom.APPROVED)

    @cached_property
    def is_visible(self):
        return self.is_published and self.is_flagged==False and self.is_rejected==False and self.is_deleted==False

    @cached_property
    def is_flagged(self):
        #if this has been manually approved, then its not flagged
        if self.moderation_status == ModerationAtom.APPROVED:
            return False
        max_flags = 1 if not hasattr(settings, 'BLOG_MAX_FLAGS') else settings.BLOG_MAX_FLAGS
        return len(self.get_flags()) >= max_flags

    @cached_property
    def is_rejected(self):
        return self.moderation_status == ModerationAtom.REJECTED

    def downvote(self, user):
        self.vote(user, UpvoteDownvoteFlagMolecule.DOWNVOTE)

    @cached_property
    def downvotes(self):
        return self.get_votes(UpvoteDownvoteFlagMolecule.DOWNVOTE)

    def get_downvotes_total(self):
        return len(self.downvotes)

    def upvote(self, user):
        self.vote(user, UpvoteDownvoteFlagMolecule.UPVOTE)

    @cached_property
    def upvotes(self):
        return self.get_votes(UpvoteDownvoteFlagMolecule.UPVOTE)

    def get_upvotes_total(self):
        return len(self.upvotes)

    def get_vote_total(self):
        return self.get_upvotes_total() - self.get_downvotes_total()


    def vote(self, user, type):
        vote_model = get_model(settings.BLOG_COMMENT_VOTE_MODEL.split('.')[0], settings.BLOG_COMMENT_VOTE_MODEL.split('.')[1])
        vote, vote_created = vote_model.objects.get_or_create(comment=self,voter=user)
        vote.type = type
        vote.save()

    def get_votes(self, type):
        vote_model = get_model(settings.BLOG_COMMENT_VOTE_MODEL.split('.')[0], settings.BLOG_COMMENT_VOTE_MODEL.split('.')[1])
        return vote_model.objects.filter(comment=self,type=type)

    def flag(self, user):
        flag_model = get_model(settings.BLOG_COMMENT_FLAG_MODEL.split('.')[0], settings.BLOG_COMMENT_FLAG_MODEL.split('.')[1])
        flag, flag_created = flag_model.objects.get_or_create(comment=self,voter=user)
        flag.type = UpvoteDownvoteFlagMolecule.FLAG
        flag.save()

    def get_flags(self):
        flag_model = get_model(settings.BLOG_COMMENT_FLAG_MODEL.split('.')[0], settings.BLOG_COMMENT_FLAG_MODEL.split('.')[1])
        return flag_model.objects.filter(comment=self)

    @property
    def can_update(self):

        max_age_minutes = 60 if not hasattr(settings, 'BLOG_COMMENT_ALLOW_EDIT_MAX_AGE_MINUTES') else settings.BLOG_COMMENT_ALLOW_EDIT_MAX_AGE_MINUTES
        if max_age_minutes == 0:
            return False

        #TODO: wonder if i should use modified_date or created_date here...
        comment_age = timezone.now() - self.created_date
        comment_age_minutes = (comment_age.days * (24*60)) + (comment_age.seconds/60)

        return comment_age_minutes <= max_age_minutes

    def attempt_to_update(self, new_content):
        
        if self.can_update:
            self.content = new_content
            self.save()
            return True

        return False

    @property
    def can_reply(self):
        max_reply_levels = 2 if not hasattr(settings, 'BLOG_COMMENT_MAX_REPLY_LEVELS') else settings.BLOG_COMMENT_MAX_REPLY_LEVELS
        return self.level < max_reply_levels

    def generate_title(self):
        words = 12 if not hasattr(settings, 'BLOG_COMMENT_AUTO_TITLE_LENGTH') else settings.BLOG_COMMENT_AUTO_TITLE_LENGTH
        return Truncator(self.content).words(words, html=True)
        

    def save(self, *args, **kwargs):
        allow_html = False if not hasattr(settings, 'BLOG_COMMENT_ALLOW_HTML') else settings.BLOG_COMMENT_ALLOW_HTML
        if not allow_html:
            self.content = strip_tags(self.content)

        max_words = 1000 if not hasattr(settings, 'BLOG_COMMENT_MAX_WORDS') else settings.BLOG_COMMENT_MAX_WORDS
        if max_words:
            self.content = Truncator(self.content).words(max_words, html=True)

        is_new = True if not self.pk else False            

        super(BlogComment, self).save(*args, **kwargs)

        if is_new:
            require_moderation = False if not hasattr(settings, 'BLOG_COMMENT_REQUIRE_MODERATION') else settings.BLOG_COMMENT_REQUIRE_MODERATION
            if require_moderation:
                self.moderation_status = ModerationAtom.SUBMITTED
            else:
                self.moderation_status = ModerationAtom.PUBLISHED
            super(BlogComment, self).save(*args, **kwargs)

    @classmethod
    def _build_comment_tree(cls, parent, level=0, list=None, branch=None):
        if not list:
            list = []

        if branch:
            list.append(branch)
        comment_children = cls.objects.filter(parent=parent,in_response_to=branch)
        for child in comment_children:
            list = cls._build_comment_tree(parent, level+1, list, child)
        return list



    @classmethod
    def get_comments_for_parent(cls,parent, include_drafts=False, include_rejected=True):
        tree = cls._build_comment_tree(parent)

        if include_drafts==False:
            tree = [item for item in tree if item.moderation_status!=ModerationAtom.SUBMITTED]
            # comments = comments.exclude(moderation_status=ModerationAtom.SUBMITTED)

        if include_rejected==False:
            tree = [item for item in tree if item.moderation_status!=ModerationAtom.REJECTED]
            # comments = comments.exclude(moderation_status=ModerationAtom.REJECTED)


        return tree

class BlogCommentVote(UpvoteDownvoteFlagMolecule):
    class Meta:
        abstract = True
    
    comment = models.ForeignKey('blog.BlogComment')

class BlogCommentFlag(UpvoteDownvoteFlagMolecule):
    class Meta:
        abstract = True
    
    comment = models.ForeignKey('blog.BlogComment')    





from django.db import models
from django.conf import settings
from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone
from django.utils.functional import cached_property

from .abstract import *
from .access import *
from .content import *

from carbon.utils.icons import ICON_CHOICES


class PersonAtom(models.Model):

    help = {
        'first_name': "",
        'last_name':"",
    }

    first_name = models.CharField(_('First name'), max_length=30, blank=True)
    middle_name = models.CharField(_('Middle name'), max_length=30, blank=True)
    last_name = models.CharField(_('Last name'), max_length=30, blank=True)
    date_of_birth = models.DateField(_('Date of Birth'), blank=True, null=True)

    
    class Meta:
        abstract = True

class StreetAddressAtom(models.Model):

    street_1 = models.CharField(_('Street Address 1'), max_length=30, blank=True, null=True)
    street_2 = models.CharField(_('Street Address 2'), max_length=30, blank=True, null=True)
    city = models.CharField(_('City'), max_length=30, blank=True, null=True)
    state = models.CharField(_('State'), max_length=30, blank=True, null=True)
    zipcode = models.CharField(_('Zipcode'), max_length=30, blank=True, null=True)   

    latitude = models.CharField(_('Latitude'), max_length=30, blank=True, null=True)   
    longitude = models.CharField(_('Longitude'), max_length=30, blank=True, null=True)    
    
    class Meta:
        abstract = True     


class PhoneContactAtom(models.Model):
    class Meta:
        abstract = True 

    home_phone = models.CharField(_('Home Phone'), max_length=255, blank=True, null=True) 
    work_phone = models.CharField(_('Work Phone'), max_length=255, blank=True, null=True) 
    cell_phone = models.CharField(_('Cell Phone'), max_length=255, blank=True, null=True)




class StreetAddressMolecule(VersionableAtom, StreetAddressAtom):
    class Meta:
        abstract = True

class UserMolecule(VersionableAtom, PersonAtom, AbstractBaseUser, PermissionsMixin):
    help = {
        'email':"",
    }

    email = models.EmailField(_('email address'), unique=True, blank=True)

    is_staff = models.BooleanField(_('staff status'), default=False,
        help_text=_('Designates whether the user can log into this admin '
                    'site.'))
    is_active = models.BooleanField(_('active'), default=True,
        help_text=_('Designates whether this user should be treated as '
                    'active. Unselect this instead of deleting accounts.'))
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)

    USERNAME_FIELD = 'email'

    @staticmethod
    def autocomplete_search_fields():
        return ("email__icontains", "first_name__icontains", "middle_name__icontains", "last_name__icontains")

    def get_short_name(self):
        if self.first_name:
            return self.first_name
        return self.email

    def get_public_name(self):
        if self.first_name:
            return self.first_name
        return 'Anonymous User'

    def get_full_name(self):
        if self.first_name and self.last_name:
            return u"%s %s" % (self.first_name, self.last_name)
        elif self.first_name:
            return u"%s (%s)" % (self.first_name, self.email)
        elif self.last_name:
            return u"%s (%s)" % (self.first_name, self.email)
        else:
            return self.email

    def __unicode__(self):
        return self.get_full_name()

    class Meta:
        abstract = True

class UserProfileMolecule(UserMolecule, HasImageAtom):

    help = {
        'about':"",
    }

    about = models.TextField(_('about'), help_text=help['about'], null=True, blank=True)


    class Meta:
        abstract = True

class SocialContactLinkMolecule( VersionableAtom, TitleAtom, OrderedItemAtom, LinkAtom):

    user = models.ForeignKey(settings.AUTH_USER_MODEL, 
        blank=True, null=True)
    
    icon = models.CharField(max_length=255, null=True, blank=True, choices=ICON_CHOICES, 
        help_text='Preview icons at http://fontawesome.io/icons/',)  
    
    class Meta:
        abstract = True


from django.conf.urls.static import static
from django.conf.urls import patterns, url, include

from .views import *

urlpatterns = patterns('',
    
    url(r'admin/media/imagepicker/$', ImagePickerView.as_view(), name="admin_imagepicker_view"), 
    
    url(r'admin/media/images/batch/$', ImageBatchView.as_view(), name="admin_image_batch_view"), 
    url(r'admin/media/secureimages/batch/$', SecureImageBatchView.as_view(), name="admin_secureimage_batch_view"), 
    url(r'admin/media/documents/batch/$', DocumentBatchView.as_view(), name="admin_document_batch_view"), 
    url(r'admin/media/securedocuments/batch/$', SecureDocumentBatchView.as_view(), name="admin_securedocument_batch_view"), 

)




from django.conf import settings

from django.http import Http404
from django.utils.translation import ugettext as _
from django.views.generic.detail import SingleObjectMixin


class PublishableView(SingleObjectMixin):

    def is_published(self):
        if not self.object:
            self.object = self.get_object()
        if not self.object:
            return False

        if self.object.is_published() == False:
            is_super_user = hasattr(self.request, 'user') and self.request.user and self.request.user.is_authenticated() and self.request.user.is_superuser
            if is_super_user==False:
                return False
        else:
            return True


    def get(self, request, *args, **kwargs):
        if self.is_published() == False:
            raise Http404(_("No published %(verbose_name)s found matching the query") %
                        {'verbose_name': self.model._meta.verbose_name})

        return super(PublishableView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):

        if self.is_published() == False:
            raise Http404(_("No published %(verbose_name)s found matching the query") %
                        {'verbose_name': self.model._meta.verbose_name})

        return super(PublishableView, self).post(request, *args, **kwargs)



class ModerationView(SingleObjectMixin):

    def is_moderated(self):
        if not self.object:
            self.object = self.get_object()
        if not self.object:
            return False
        if self.object.is_moderated() == False:
            is_super_user = hasattr(self.request, 'user') and self.request.user and self.request.user.is_authenticated() and self.request.user.is_superuser
            if is_super_user==False:
                return False
        else:
            return True


    def get(self, request, *args, **kwargs):
        if self.is_moderated() == False:
            raise Http404(_("No moderated %(verbose_name)s found matching the query") %
                        {'verbose_name': self.model._meta.verbose_name})

        return super(ModerationView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):

        if self.is_moderated() == False:
            raise Http404(_("No moderated %(verbose_name)s found matching the query") %
                        {'verbose_name': self.model._meta.verbose_name})

        return super(ModerationView, self).post(request, *args, **kwargs)



from django import forms
from django.conf import settings
from django.contrib import messages
from django.template import Context, Template
from django.utils.translation import ugettext_lazy as _
from django.template.defaultfilters import slugify
from django.core.context_processors import csrf

from ckeditorfiles.widgets import CKEditorWidget

from .models import *

blog_article_model = get_model(settings.BLOG_ARTICLE_MODEL.split('.')[0], settings.BLOG_ARTICLE_MODEL.split('.')[1])
blog_comment_model = get_model(settings.BLOG_COMMENT_MODEL.split('.')[0], settings.BLOG_COMMENT_MODEL.split('.')[1])

class BlogCommentForm(forms.Form):

    ACTION_SUBMIT = 'submit'
    ACTION_REPLY = 'reply'
    ACTION_UPDATE = 'update'
    ACTION_DELETE = 'delete'
    ACTION_UPVOTE = 'upvote'
    ACTION_DOWNVOTE = 'downvote'
    ACTION_FLAG = 'flag'

    BLOG_COMMENT_ACTION_CHOICES = (
        (ACTION_SUBMIT, _("Submit")),
        (ACTION_REPLY, _("Reply")),
        (ACTION_UPDATE, _("Update")),
        (ACTION_DELETE, _("Delete")),
        (ACTION_UPVOTE, _("Upvote")),
        (ACTION_DOWNVOTE, _("Downvote")),
        (ACTION_FLAG, _("Flag")),
    )


    

    comment_action = forms.ChoiceField(required=True,choices=BLOG_COMMENT_ACTION_CHOICES, widget=forms.HiddenInput())
    content = forms.CharField(required=False, widget=forms.Textarea(attrs={'placeholder': _("Comment")}), label=_("Comment"))
    article = forms.ModelChoiceField(queryset=blog_article_model.objects.all(), required=False, widget=forms.HiddenInput())
    reply = forms.ModelChoiceField(queryset=blog_comment_model.objects.all(), required=False, widget=forms.HiddenInput())
    
    def __init__( self, request, was_posted, *args, **kwargs ):
        super( BlogCommentForm, self ).__init__( *args, **kwargs )
        self.request = request
        self.was_posted = was_posted

    def clean_comment_action(self):
        
        #Require logged in user
        if not self.request.user or not self.request.user.is_authenticated():
            raise forms.ValidationError(_("Please log in to comment"))

        return self.data['comment_action']


    def clean_content(self):
        
        comment_action = self.data['comment_action']
        comment_content = self.cleaned_data['content']
        
        if not self.was_posted:
            return comment_content
        # raise forms.ValidationError(_("This is a validation test"))

        if comment_action == BlogCommentForm.ACTION_SUBMIT:
            if not comment_content:
                raise forms.ValidationError(_("Please enter a comment"))

        elif comment_action == BlogCommentForm.ACTION_REPLY:
            if not self.cleaned_data['content']:
                raise forms.ValidationError(_("Please enter a comment"))

        elif comment_action == BlogCommentForm.ACTION_UPDATE:
            if not self.cleaned_data['content']:
                raise forms.ValidationError(_("Please enter a comment"))

        return comment_content


    def clean_reply(self):
        
        comment_action = self.data['comment_action']
        comment_reply_to_comment = self.cleaned_data['reply']
        
        if not self.was_posted:
            return comment_reply_to_comment

        if comment_action == BlogCommentForm.ACTION_REPLY:
            if not comment_reply_to_comment:
                raise forms.ValidationError(_("Please enter a comment to reply to"))

            if not comment_reply_to_comment.can_reply:
                raise forms.ValidationError(_("You cannot reply to this comment"))  

        elif comment_action == BlogCommentForm.ACTION_UPDATE:
            if not comment_reply_to_comment:
                raise forms.ValidationError(_("Please enter a comment to update"))

        elif comment_action == BlogCommentForm.ACTION_FLAG:
            if not comment_reply_to_comment:
                raise forms.ValidationError(_("Please enter a comment to flag"))

        elif comment_action == BlogCommentForm.ACTION_UPVOTE:
            if not comment_reply_to_comment:
                raise forms.ValidationError(_("Please enter a comment to upvote"))

        elif comment_action == BlogCommentForm.ACTION_DOWNVOTE:
            if not comment_reply_to_comment:
                raise forms.ValidationError(_("Please enter a comment to downvote"))

        elif comment_action == BlogCommentForm.ACTION_DELETE:
            if not comment_reply_to_comment:
                raise forms.ValidationError(_("Please enter a comment to delete"))

        return comment_reply_to_comment



    def clean_article(self):
     
        comment_action = self.data['comment_action']
        comment_article = self.cleaned_data['article']

        if not self.was_posted:
            return comment_article
        
        if comment_action == BlogCommentForm.ACTION_SUBMIT:
            if not comment_article:
                raise forms.ValidationError(_("Please enter an article to comment on"))

        return comment_article

    def save(self):
        comment_action = self.data['comment_action']
        
        if comment_action == BlogCommentForm.ACTION_SUBMIT:
            return self.submit_comment()

        elif comment_action == BlogCommentForm.ACTION_REPLY:
            return self.reply_to_comment()

        elif comment_action == BlogCommentForm.ACTION_UPDATE:
            return self.update_comment()
        
        elif comment_action == BlogCommentForm.ACTION_DELETE:
            return self.delete_comment()


        elif comment_action == BlogCommentForm.ACTION_FLAG:
            return self.flag_comment()

        elif comment_action == BlogCommentForm.ACTION_UPVOTE:
            return self.upvote_comment()

        elif comment_action == BlogCommentForm.ACTION_DOWNVOTE:
            return self.downvote_comment()

        
        return data


    def submit_comment(self):
        
        comment = blog_comment_model(
            user = self.request.user,
            content=self.cleaned_data['content'],
            article=self.cleaned_data['article']
        )
        comment.save()

        messages.success(self.request, _("Your comment has been submitted"))

        return comment

    def reply_to_comment(self):
        
        comment = blog_comment_model(
            user = self.request.user,
            content=self.cleaned_data['content'],
            article=self.cleaned_data['article'],
            in_response_to=self.cleaned_data['reply']
        )
        comment.save()

        messages.success(self.request, _("Your reply has been submitted"))

        return comment

    def update_comment(self):
        current_comment = self.cleaned_data['reply']
        updated = current_comment.attempt_to_update(self.cleaned_data['content'])
        if updated:
            messages.success(self.request, _("Your comment has been updated"))
        else:
            messages.success(self.request, _("Your comment can no longer be updated"))

        return current_comment

    def delete_comment(self):
        reply = self.cleaned_data['reply']
        reply.is_deleted = True
        reply.save()
        messages.success(self.request, _("Your comment has been deleted"))
        return reply


    def upvote_comment(self):
        comment = self.cleaned_data['reply']
        comment.upvote(self.request.user)

        messages.success(self.request, _("Thanks for the feedback! This comment has been up-voted."))

    def downvote_comment(self):
        comment = self.cleaned_data['reply']
        comment.downvote(self.request.user)

        messages.success(self.request, _("Thanks for the feedback! This comment has been down-voted."))

    def flag_comment(self):
        comment = self.cleaned_data['reply']
        comment.flag(self.request.user)

        messages.success(self.request, _("Thanks for the feedback! This comment has been flagged to be moderated."))

    @classmethod
    def create_submit_form(cls, context, form_template, article):
        return cls._create_comment_form(context, form_template, BlogCommentForm.ACTION_SUBMIT, article, None )      

    @classmethod
    def create_reply_form(cls, context, form_template, comment):
        return cls._create_comment_form(context, form_template, BlogCommentForm.ACTION_REPLY, comment.article, comment)

    @classmethod
    def create_update_form(cls, context, form_template, comment):
        return cls._create_comment_form(context, form_template, BlogCommentForm.ACTION_UPDATE, comment.article, comment)

    @classmethod
    def create_delete_form(cls, context, form_template, comment):
        return cls._create_comment_form(context, form_template, BlogCommentForm.ACTION_DELETE, comment.article, comment)

    @classmethod
    def create_flag_form(cls, context, form_template, comment):
        return cls._create_comment_form(context, form_template, BlogCommentForm.ACTION_FLAG, comment.article, comment)

    @classmethod
    def create_upvote_form(cls, context, form_template, comment):
        return cls._create_comment_form(context, form_template, BlogCommentForm.ACTION_UPVOTE, comment.article, comment)

    @classmethod
    def create_downvote_form(cls, context, form_template, comment):
        return cls._create_comment_form(context, form_template, BlogCommentForm.ACTION_DOWNVOTE, comment.article, comment)

    @classmethod
    def _create_comment_form(cls, context, form_template, method, article, comment=None):
        request = context['request']

        content_ediable_actions = [
            BlogCommentForm.ACTION_SUBMIT,
            BlogCommentForm.ACTION_UPDATE,
            BlogCommentForm.ACTION_REPLY
        ]
        if method in content_ediable_actions:
            widgets = {'content':forms.Textarea(attrs={'placeholder': _("Comment")})}    
        else:
            widgets = {'content':forms.HiddenInput()}

        initial_values = {}
        initial_values['comment_action'] = method
        initial_values['article'] = article
        
        if method == BlogCommentForm.ACTION_UPDATE:
            initial_values['content'] = comment.content
        else:
            initial_values['content'] = ""

        if comment:
            initial_values['reply'] = comment

        data = None
        was_posted = False

        if request.POST:
            was_posted = request.POST['comment_action'] == method
            is_related_form = True if not comment else str(comment.pk) == str(request.POST['reply'])
            data = {}
            data['article'] = article.pk
            if comment:
                data['reply'] = comment.pk
            data['comment_action'] = method
            data['content'] = request.POST['content'] if (was_posted and is_related_form) else ""        
            
        button_label = dict(BlogCommentForm.BLOG_COMMENT_ACTION_CHOICES).get(method)
        form_id = "comment-%s-form"%(method) if not comment else "comment-%s-%s-form"%(method, comment.pk)
        return cls.create_and_render_form(request, (was_posted and is_related_form), form_template, initial_values, widgets, button_label, form_id, data)

    
    @classmethod
    def create_and_render_form(cls, request, was_posted, form_template, initial_values, widgets, submit_label, form_id, data=None, files=None):

        form = cls.create_form(request, was_posted, form_template, initial_values, widgets, data, files)
        return cls.render_form(form, request, submit_label, form_id)

    @classmethod
    def create_form(cls, request, was_posted, form_template, initial_values, widgets, data=None, files=None):
        kwargs = {
            'initial': initial_values,
            'prefix': None,
        }
        kwargs['request'] = request
        kwargs['was_posted'] = was_posted

        if data:
            kwargs['data'] = data

        if files:
            kwargs['files'] = files
        

        new_form = cls(**kwargs)
        
        for widget_key, widget in widgets.iteritems():
            new_form.fields[widget_key].widget = widget

        #Now set initial values:
        if data:
            for data_key, data in data.iteritems():
                new_form.fields[data_key].initial = data




        return new_form

    @classmethod
    def render_form(cls, form, request, submit_label, form_id):
            
        template_string = '\
        <form action="" method="post" class="{{form_class}}" id="{{form_id}}">{% csrf_token %}\
            {{ form.as_ul }}\
            <input type="submit" value="{{submit_label}}">\
        </form>'

        template = Template(template_string)
        context_dict = {}
        context_dict.update(csrf(request))
        context_dict['form'] = form
        context_dict['submit_label'] = submit_label
        error_class = '' if not form.was_posted else 'valid' if bool(form._errors) else 'invalid'
        context_dict['form_class'] = "%s %s"%(error_class, slugify(submit_label))
        context_dict['form_id'] = form_id
        context = Context(context_dict)
        return template.render(context)
    

#=============================================
# Admin Forms ================================
#=============================================

class BlogArticleAdminForm(forms.ModelForm):
    content = forms.CharField(widget=CKEditorWidget(config=settings.CKEDITOR_CONFIGS['blogarticle_content_ckeditor']), required=False)
    synopsis = forms.CharField(widget=CKEditorWidget(config=settings.CKEDITOR_CONFIGS['blogarticle_synopsis_ckeditor']), required=False)
    class Meta:
        model = BlogArticle
        fields = '__all__'

class BlogCategoryAdminForm(forms.ModelForm):
    content = forms.CharField(widget=CKEditorWidget(config=settings.CKEDITOR_CONFIGS['blogcategory_content_ckeditor']), required=False)
    synopsis = forms.CharField(widget=CKEditorWidget(config=settings.CKEDITOR_CONFIGS['blogcategory_synopsis_ckeditor']), required=False)
    class Meta:
        model = BlogCategory
        fields = '__all__'

class BlogTagAdminForm(forms.ModelForm):
    content = forms.CharField(widget=CKEditorWidget(config=settings.CKEDITOR_CONFIGS['blogtag_content_ckeditor']), required=False)
    synopsis = forms.CharField(widget=CKEditorWidget(config=settings.CKEDITOR_CONFIGS['blogtag_synopsis_ckeditor']), required=False)
    class Meta:
        model = BlogTag
        fields = '__all__'





import cStringIO
from PIL import Image
import re
import ast
from dateutil.parser import parse as parsedate

from django import forms
from django.db import models
from django.db.models import Q
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.core.urlresolvers import reverse_lazy
from django.core.validators import validate_email, URLValidator
from django.utils.translation import ugettext_lazy as _



from carbon.utils.slugify import unique_slugify
from carbon.utils.icons import ICON_CHOICES

from carbon.atoms.models.abstract import *
from carbon.atoms.models.content import *

from carbon.atoms.models.media import BaseSecureAtom

from .widgets import *
from .utils import *


# class FormClass(VersionableAtom, TitleAtom):
#   class Meta:
#       abstract = True




class Form(ContentMolecule):
    field_model = None
    help = {
        
        'admin_email':"",
        'email_admin_override':"Separate email addresses with comma, semi-color or space. Leave blank to send to default email address (%s)"%(settings.DEFAULT_FROM_EMAIL),
        'email_admin_on_submission':"",
        'email_admin_on_submission_template':"",
        'email_admin_on_submission_category':"",
        'email_user_field_slug':"Enter the slug of the field that should be used to determine the user's email address",
        'email_user_on_submission_template':"",
        'email_user_on_submission_category':"",
        'submission_content':"",
        'submit_template':"",

        'redirect_url_on_submission':"When a form is submitted you may override where the user is redirected.",
        'form_action': "Defines whether to display this form on its own page with its own URL, or whether to embed it on another page elsehwere in the site. NOTE: Several of the subsections below only apply if the form action is a standalone form.",
        'required_logged_in_user':"Requires user to log in or create an account before filling out form. NOTE: This should only be turned on if you have enabled user registration on the site.",
        'is_editable':"Allows user to update the entry. NOTE: If this is checked, unless you also require a logged in user on the form, anyone with the correct URL can later update the entry. Therefore it is recommended that you use this in conjunction with requiring a logged in user.",
        
        'submit_label':"Label on the submit button.",
        'form_error_message':"Global message to show user when there is an error in the form. NOTE: Individual fields have separate error messages.",
        'form_create_message':"Message to show user when they successfully submit the form.",
        'form_update_message':"Message to show user when they successfully update the form. NOTE: Form must be editable to allow users to update the form.",

        'extra_css_classes':"Adds custom css classes into the form template.",
        'third_party_id': "An identifier to integrate the form with another system",
    }

    POST_TO_FORM_PAGE = 'form-page'
    POST_TO_EMBEDDED_PAGE = 'embedded-page'
    FORM_ACTION_CHOICES = (
        (POST_TO_FORM_PAGE, _("Standalone Form")),
        (POST_TO_EMBEDDED_PAGE, _("Form Embedded in Page"))
    )

    form_action = models.CharField(max_length=255, choices=FORM_ACTION_CHOICES, 
        default=POST_TO_FORM_PAGE, help_text=help['form_action'])

    required_logged_in_user = models.BooleanField(default=False, 
        help_text=help['required_logged_in_user'])

    is_editable = models.BooleanField(default=False, 
        help_text=help['is_editable'])


    email_admin_override = models.CharField(_('Admins to email on submission'), null=True, blank=True, 
        max_length=255,help_text=help['email_admin_override'])
    email_admin_on_submission = models.BooleanField(default=True, 
        help_text=help['email_admin_on_submission'])
    email_admin_on_submission_template = models.ForeignKey(settings.EMAIL_TEMPLATE_MODEL, 
        null=True, blank=True, help_text=help['email_admin_on_submission_template'],
       related_name='email_admin_on_submission_template')
    admin_email = models.EmailField(max_length=255, blank=True, null=True,
        help_text=help['admin_email'])
    email_admin_on_submission_category = models.ForeignKey(settings.EMAIL_CATEGORY_MODEL, 
        null=True, blank=True, help_text=help['email_admin_on_submission_category'],
        related_name='email_admin_on_submission_category')


    email_user_field_slug = models.CharField(null=True, blank=True, 
        max_length=255,help_text=help['email_user_field_slug'])
    email_user_on_submission = models.BooleanField(default=True, 
        help_text=help['email_admin_on_submission'])
    email_user_on_submission_template = models.ForeignKey(settings.EMAIL_TEMPLATE_MODEL, 
        null=True, blank=True, help_text=help['email_user_on_submission_template'],
        related_name='email_user_on_submission_template')
    email_user_on_submission_category = models.ForeignKey(settings.EMAIL_CATEGORY_MODEL, 
        null=True, blank=True, help_text=help['email_user_on_submission_category'],
        related_name='email_user_on_submission_category')

    redirect_url_on_submission = models.CharField(max_length=255, null=True, 
        blank=True, help_text=help['redirect_url_on_submission'])

    submit_template = models.ForeignKey(settings.TEMPLATE_MODEL, null=True, blank=True,
        help_text=help['submit_template'], related_name='template_submit_template')   

    submission_content = models.TextField(help_text=help['submission_content'], 
        null=True, blank=True)

    submit_label = models.CharField(max_length=255, default="Submit",
        help_text=help['submit_label'])


    form_error_message = models.CharField(max_length=255, blank=True, null=True,
        help_text=help['form_error_message'])
    form_create_message = models.CharField(max_length=255, blank=True, null=True,
        help_text=help['form_create_message'])
    form_update_message = models.CharField(max_length=255, blank=True, null=True,
        help_text=help['form_update_message'])



    extra_css_classes = models.CharField(max_length=255, null=True, blank=True, 
        help_text=help['extra_css_classes'])

    third_party_id = models.CharField(max_length=255, blank=True, null=True,
        help_text=help['third_party_id'])

    def get_success_url(self, entry=None):
        if self.redirect_url_on_submission:
            return self.redirect_url_on_submission
        else:
            if self.is_editable and entry != None:
                return entry.get_absolute_url()
            else:
                return reverse('form_submitted_view', kwargs = {'path': self.slug }) 

    def handle_successful_submission(self, form, entry, created):
        if created:
            
            input_fields = self.get_input_fields()
            fields = []
            fields_dict = {}

            field_entry_hash = {}
            field_entries = entry.get_entries()
            for field_entry in field_entries:
                field_entry_hash[field_entry.form_field.slug] = field_entry

            #POPULATE KEY/VALUE PAIR:
            for field in input_fields:
                field_entry = field_entry_hash[field.slug]
                object = {
                    'title':field.title,
                    'value':field_entry.value,
                    'rendered_value':field_entry.get_rendered_value(),
                    'is_honeypot':field.is_honeypot,
                    'entry':field_entry, 
                    'field':field
                }
                fields.append(object)
                fields_dict[field.slug] = object

            context = {
                'input_fields':input_fields,
                'all_fields':self.get_all_fields(),
                'submitted_fields':fields,
                'submitted_fields_dict':fields_dict,
                'entry':entry,
                'form':self
            }

            if self.email_admin_on_submission:

                if not self.email_admin_on_submission_template:
                    raise ImproperlyConfigured( "Form is set to email admin on submission, but email admin template not specified" )

                if not self.email_admin_on_submission_category:
                    raise ImproperlyConfigured( "Form is set to email admin on submission, but email admin category not specified" )

                admin_email_list = self.get_admin_emails()
                for admin_email in admin_email_list:
                    self.email_admin_on_submission_template.send_email_message(admin_email, self.email_admin_on_submission_category, context)
            
            if self.email_user_on_submission:
                user_email_field = self.get_email_recipient_field()
                if user_email_field==None:
                    raise ImproperlyConfigured( "User recipient field not found" )
                
                user_email_entry = entry.get_entry_by_slug(user_email_field.slug)
                if user_email_entry:
                    if not self.email_user_on_submission_template:
                        raise ImproperlyConfigured( "Form is set to email user on submission, but email user template not specified" )

                    if not self.email_user_on_submission_category:
                        raise ImproperlyConfigured( "Form is set to email user on submission, but email user category not specified" )

                    self.email_user_on_submission_template.send_email_message(user_email_field, 
                            self.email_user_on_submission_category, context)

    def get_admin_emails(self):
        if self.email_admin_override:
            return re.split('; |, ',self.email_admin_override)
        return [settings.DEFAULT_FROM_EMAIL]

    def get_field_by_slug(self, slug):
        try:
          return self.field_model.objects.get(parent=self,slug=slug)
        except:
          return None

    def get_all_fields(self):
        return self.field_model.objects.filter(parent=self).exclude(hide=True).order_by('order')

    def get_input_fields(self):
        return self.field_model.objects.filter(parent=self).exclude(hide=True).exclude(
            Q(type=FormField.FORM_INSTRUCTIONS) | 
            Q(type=FormField.FORM_DIVIDER) |
            Q(type=FormField.FORM_STEP)
        ).order_by('order')

    def get_email_recipient_field(self):
        #TODO -- allow admin to specify which field is the email field

        if self.email_user_field_slug:
            fields = self.get_all_fields()
            email_fields = fields.filter(slug=self.email_user_field_slug)
            if len(email_fields) > 0:
                return email_fields[0]
        

        email_fields = self.field_model.objects.filter(parent=self).exclude(hide=True).filter(
            type=FormField.EMAIL_FIELD
        ).order_by('order')
        if len(email_fields) > 0:
            return email_fields[0]
        
        return None

    def has_multipart(self):
        fields = self.get_all_fields().filter(
            Q(type=FormField.FILE) | 
            Q(type=FormField.SECURE_FILE) | 
            Q(type=FormField.IMAGE)
        )
        return len(fields) > 0

    def get_form_action(self):
        if self.form_action == Form.POST_TO_FORM_PAGE:
            return self.get_absolute_url()
        else:
            return '.'

    def get_absolute_url(self):
        if self.form_action == Form.POST_TO_FORM_PAGE:
            return reverse('form_create_view', kwargs = {'path': self.slug }) 
        else:
            return None
        

    def __unicode__(self):
        return self.title

    class Meta:
        verbose_name_plural = "Forms"
        verbose_name = "Forms"
        abstract = True



class Validation(models.Model):
    #Maps to http://parsleyjs.org/doc/index.html#psly-validators-overview
    help = {
        'is_required':"If this field is required, a value of some sort is needed for the user to submit the form. See the advanced validation options to apply more specific validation parameters.",
        'is_digits':"",
        'is_alphanumeric':"",
        'min_length':"",
        'max_length':"",
        'min_words':"",
        'max_words':"",
        'min_value':"",
        'max_value':"",
        'min_check':"",
        'max_check':"",     
        'min_date':"",
        'max_date':"",
        'min_datetime':"",
        'max_datetime':"",
        'min_width':"Applies to image uploads",
        'max_width':"Applies to image uploads",
        'min_height':"Applies to image uploads",
        'max_height':"Applies to image uploads",
        'min_size':"Applies to image and file uploads, measured in MB; e.g. 5000 is 5GB, 0.5 is 500KB",
        'max_size':"Applies to image and file uploads, measure in MB; e.g. 5000 is 5GB, 0.5 is 500KB",
        'pattern':"Match a value or validate file types (e.g. .*\.txt|.*\.pdf|.*\.doc)",
        'equal_to':"Enter form field slug that this field should match",
        'step_interval':"If values is a score, range, or slider, pick the step interval."
    }


    
    is_required = models.BooleanField(default=False, 
        help_text=help['is_required'])
    is_digits = models.BooleanField(default=False, 
        help_text=help['is_digits'])
    is_alphanumeric = models.BooleanField(default=False, 
        help_text=help['is_alphanumeric'])


    min_length = models.IntegerField(null=True, blank=True, help_text=help['min_length']) 
    max_length = models.IntegerField(null=True, blank=True, help_text=help['max_length']) 
    min_words = models.IntegerField(null=True, blank=True, 
        help_text=help['min_words'])
    max_words = models.IntegerField(null=True, blank=True, 
        help_text=help['max_words'])


    min_value = models.IntegerField(null=True, blank=True, help_text=help['min_value']) 
    max_value = models.IntegerField(null=True, blank=True, help_text=help['max_value'])

    min_check = models.IntegerField(null=True, blank=True, help_text=help['min_check']) 
    max_check = models.IntegerField(null=True, blank=True, help_text=help['max_check'])  
    

    min_date = models.DateField(blank=True, null=True, 
        help_text=help['min_date'])
    max_date = models.DateField(blank=True, null=True, 
        help_text=help['max_date'])
    min_datetime = models.DateTimeField(blank=True, null=True, 
        help_text=help['min_datetime'])
    max_datetime = models.DateTimeField(blank=True, null=True, 
        help_text=help['max_datetime'])

    min_width = models.IntegerField(null=True, blank=True, 
        help_text=help['min_width'])
    max_width = models.IntegerField(null=True, blank=True, 
        help_text=help['max_width'])
    min_height = models.IntegerField(null=True, blank=True, 
        help_text=help['min_height'])
    max_height = models.IntegerField(null=True, blank=True, 
        help_text=help['max_height'])
    min_size = models.IntegerField(null=True, blank=True, 
        help_text=help['min_size'])
    max_size = models.IntegerField(null=True, blank=True, 
        help_text=help['max_size'])

    
    step_interval = models.DecimalField(null=True, blank=True, help_text=help['step_interval'],
        max_digits=9, decimal_places=2) 

    pattern = models.CharField(max_length=255, null=True, blank=True, 
        help_text=help['pattern'])
    pattern_error_message = models.CharField(max_length=255, null=True, blank=True)

    equal_to = models.CharField(max_length=255, null=True, blank=True, 
        help_text=help['equal_to'])
    equal_to_error_message = models.CharField(max_length=255, null=True, blank=True)


    def validate(self, value, form):
        if value:
            # if self.is_required and not value:
            #     raise ValidationError(_("This field is required"))  

            if self.type == FormField.HONEYPOT_FIELD:
                raise ValidationError(_("Please try again"))

            if self.type == FormField.INTEGER_FIELD and not is_int(value):
                raise ValidationError(_("Please enter an integer"))

            # if self.type == FormField.NUMBER_FIELD and not value.isdigit():
            #     raise ValidationError(_("Please enter a number"))
        
            if self.is_digits and not value.isdigit():
              raise ValidationError(_("Please enter numeric digits only"))

            if self.is_alphanumeric and not value.isalnum():
              raise ValidationError(_("Please enter leters and numbers only"))

            if self.min_length and not len(value) >= self.min_length:
              raise ValidationError(_("Please enter at least %s characters"%(self.min_length)))

            if self.max_length and not len(value) <= self.max_length:
              raise ValidationError(_("Please enter no more than %s characters"%(self.max_length)))

            if self.min_words and not len(value.split()) >= self.min_words:
              raise ValidationError(_("Please enter at least %s words"%(self.min_words)))

            if self.max_words and not len(value.split()) <= self.max_words:
              raise ValidationError(_("Please enter no more than %s characters"%(self.max_words)))

            if self.min_value and not float(value) >= self.min_value:
              raise ValidationError(_("Please enter a value of %s or greater"%(self.min_value)))

            if self.max_value and not float(value) <= self.max_value:
              raise ValidationError(_("Please enter a value of %s or less"%(self.max_value)))

            if self.type == FormField.DATE or self.type == FormField.DATE_TIME:
              date = parsedate(value).date()
              if self.min_date and not date >= self.min_date:
                raise ValidationError(_("Please enter a date of %s or later"%(self.min_date)))
              if self.max_date and not date <= self.max_date:
                raise ValidationError(_("Please enter a date of %s or earlier"%(self.max_date)))

                if self.type == FormField.DATE_TIME:
                    if self.min_datetime and not date >= self.min_datetime:
                      raise ValidationError(_("Please enter a date and time of %s or later"%(self.min_datetime)))
                    if self.max_datetime and not date <= self.max_datetime:
                      raise ValidationError(_("Please enter a date and time of %s or earlier"%(self.max_datetime)))

            if self.type == FormField.EMAIL_FIELD:
                validate_email(value)

            if self.type == FormField.URL_FIELD:
                validator = URLValidator()
                validator(value)





            if self.is_multipart and value:
              
              if self.min_size and not (self.min_size * 1024 * 1024) <= value.size:
                raise ValidationError(_("Please select a file that is %sMB or larger"%(self.min_size)))

              if self.max_size and not (self.max_size * 1024 * 1024) >= value.size:
                raise ValidationError(_("Please select a file that is %sMB or smaller"%(self.max_size)))

            # if self.type==FormField.IMAGE and value:
              

            #   if self.min_width or self.max_width or self.min_height or self.max_height:

            #     if isinstance(value, TemporaryUploadedFile):
            #       temp_file = open(value.temporary_file_path(), 'rb+')
            #       content = cStringIO.StringIO(temp_file.read())
            #       image = Image.open(content)
            #       width, height = image.size
            #     else:
            #       # print value.read()
            #       # print value.file.read()
            #       # content = cStringIO.StringIO(value.read())
            #       image = Image.open(value.read())
            #       width, height = image.size

            #     if self.min_width and not self.min_width <= width:
            #       raise ValidationError(_("Please select an image that is %spx wide or more"%(self.min_width)))

            #     if self.max_width and not self.max_width >= width:
            #       raise ValidationError(_("Please select an image that is %spx wide or less"%(self.max_width)))

            #     if self.min_height and not self.min_height <= height:
            #       raise ValidationError(_("Please select an image that is %spx tall or more"%(self.min_height)))

            #     if self.max_height and not self.max_height >= height:
            #       raise ValidationError(_("Please select an image that is %spx tall or less"%(self.max_height)))
                


            if self.equal_to:
              key = 'form_field_%s'%(self.equal_to)
              equal_value = form._raw_value(key)
              if not equal_value:
                field = self.form.get_field_by_slug(key)
                if field:
                    equal_value = field.default
              if not value == equal_value:
                raise ValidationError(self.equal_to_error_message)

            if self.pattern:
              if value:
                  
                test = value
                if isinstance(value, InMemoryUploadedFile) or isinstance(value, TemporaryUploadedFile):
                  test = value.name

                # print self.pattern
                # print test
                # print self.pattern.decode('string_escape')

                result = re.match(self.pattern, test)
                if not result:
                  raise ValidationError(self.pattern_error_message)



    class Meta:

        abstract = True

class FormField(VersionableAtom, TitleAtom, Validation):

    help = {
        'order': "",
        'type':"Fill in choices field (in advanced options) for select fields. Fill in content field for instructions. WARNING: Only use password and secure file in conjunction with HTTPS.",
        'secondary_label':"",
        'placeholder_text':"",
        'help_text':"",
        'content':"Rich-text instructions",
        'default':"Default field value",
        'extra_css_classes':"Adds custom css classes onto the form field in the template.",
        'hide':"Hide field from form without deleting and data entered by users. Use this instead of deleting a form field.",
        'icon_right':'Add icon to the right side of the field. Preview icons at http://fontawesome.io/icons/',
        'icon_left':'Add icon to the left side of the field. Preview icons at http://fontawesome.io/icons/',
        'inset_text_right':"Inset field with content on the right",
        'inset_text_left':"Inset field with content on the left",
        'error_message':"Message to display when this field is invalid.",
        'third_party_id': "An identifier to integrate the form with another system",
        'choices':"Comma separated options where applicable. If an option "
            "itself contains commas, surround the option starting with the %s"
            "character and ending with the %s character." %
                (settings.FORM_CHOICES_QUOTE, settings.FORM_CHOICES_UNQUOTE)
    }



    TEXT_FIELD = 'text-field'
    EMAIL_FIELD = 'email-field'
    URL_FIELD = 'url-field'
    INTEGER_FIELD = 'integer-field'
    NUMBER_FIELD = 'number-field'
    TEXT_AREA = 'text-area'
    BOOLEAN_CHECKBOXES = 'boolean-checkboxes'
    BOOLEAN_TOGGLE = 'boolean-toggle'
    SELECT_DROPDOWN = 'select-dropdown'
    SELECT_RADIO_BUTTONS = 'select-radio-buttons'
    SELECT_BUTTONS = 'select-buttons'
    SELECT_IMAGE = 'select-image'
    SELECT_MULTIPLE_CHECKBOXES = 'select-multiple-checkboxes'
    SELECT_MULTIPLE_AUTOSUGGEST = 'select-multiple-autosuggest'
    SELECT_MULTIPLE_HORIZONTAL = 'select-multiple-horizontal'
    SELECT_MULTIPLE_BUTTONS = 'select-multiple-buttons'
    SELECT_MULTIPLE_IMAGES = 'select-multiple-images'
    COMMA_SEPARATED_LIST = 'comma-separated-list'
    FILE = 'file'
    SECURE_FILE = 'secure-file'
    IMAGE = 'image'
    DATE = 'date'
    TIME = 'time'
    DATE_TIME = 'date-time'
    SCORE = 'score'
    RANGE = 'range'
    NUMBER_SLIDER = 'number-slider'
    PASSWORD = 'password'

    FORM_INSTRUCTIONS = 'form-instructions'
    FORM_DIVIDER = 'form-divider'
    FORM_STEP = 'form-step'
    HIDDEN_FIELD = 'hidden-field'
    HONEYPOT_FIELD = 'honeypot-field'

        
    FORM_FIELD_CHOICES = (
        (TEXT_FIELD, _("Single Line Text Field")),
        (EMAIL_FIELD, _("Email Field")),
        (URL_FIELD, _("URL Field")),
        (INTEGER_FIELD, _("Integer Field")),
        (NUMBER_FIELD, _("Number Field")),
        (TEXT_AREA, _("Multiple Lines Text Area")),
        (BOOLEAN_CHECKBOXES, _("Single Checkbox")),
        (BOOLEAN_TOGGLE, _("Toggle")),
        (SELECT_DROPDOWN, _("Select with Dropdown")),
        (SELECT_RADIO_BUTTONS, _("Select with Radio Buttons")),
        (SELECT_BUTTONS, _("Select with Buttons")),
        (SELECT_IMAGE, _("Select Image")),
        (SELECT_MULTIPLE_CHECKBOXES, _("Select Multiple with Checkboxes")),
        (SELECT_MULTIPLE_AUTOSUGGEST, _("Select Multiple with Autosuggest")),
        (SELECT_MULTIPLE_HORIZONTAL, _("Select Multiple with Horizontal Lists")),   
        (SELECT_MULTIPLE_BUTTONS, _("Select Multiple with Buttons")),
        (SELECT_MULTIPLE_IMAGES, _("Select Multiple Images")),  
        (COMMA_SEPARATED_LIST, _("List of Items")),  
        (FILE, _("File")),
        (SECURE_FILE, _("Secure File")),
        (IMAGE, _("Image")),
        (DATE, _("Date")),
        (TIME, _("Time")),
        (DATE_TIME, _("Date and Time")),
        (SCORE, _("Score")),
        (RANGE, _("Range")),
        (NUMBER_SLIDER, _("Number on a Slider")),
        (PASSWORD, _("Password")),
        (FORM_INSTRUCTIONS, _("Form Instructions")),
        (FORM_DIVIDER, _("Form Divider")),
        (FORM_STEP, _("Form Step")),
        (HIDDEN_FIELD, _("Hidden Field")),
        (HONEYPOT_FIELD, _("Honeypot Field"))
    )

    type = models.CharField(max_length=255, choices=FORM_FIELD_CHOICES, 
        help_text=help['type'])

    parent = models.ForeignKey('form.Form', null=True, blank=True)

    order = models.IntegerField(default=0, help_text=help['order']) 

    secondary_label = models.CharField(max_length=255, null=True, 
        blank=True, help_text=help['secondary_label'])
    placeholder_text = models.CharField(max_length=255, null=True, 
        blank=True, help_text=help['placeholder_text'])
    help_text = models.CharField(max_length=255, null=True, 
        blank=True, help_text=help['help_text'])

    content = models.TextField(null=True, blank=True, help_text=help['content'])
    
    choices = models.TextField(null=True, blank=True, help_text=help['choices'])

    default = models.CharField(max_length=255, null=True, blank=True, 
        help_text=help['default'])

    extra_css_classes = models.CharField(max_length=255, null=True, blank=True, 
        help_text=help['extra_css_classes'])

    icon_right = models.CharField(max_length=255, null=True, blank=True, 
        choices=ICON_CHOICES, help_text=help['icon_right'])
    icon_left = models.CharField(max_length=255, null=True, blank=True, 
        choices=ICON_CHOICES, help_text=help['icon_left'])
    inset_text_right = models.CharField(max_length=255, null=True, blank=True, 
        help_text=help['inset_text_right'])
    inset_text_left = models.CharField(max_length=255, null=True, blank=True, 
        help_text=help['inset_text_left'])

    hide = models.BooleanField(default=False, help_text=help['hide'])

    error_message = models.CharField(max_length=255, blank=True, null=True,
        help_text=help['error_message'])

    third_party_id = models.CharField(max_length=255, blank=True, null=True,
        help_text=help['third_party_id'])

    def get_choices(self):
        raw_choices = self.choices.split(',')
        choices = tuple((n,n) for n in raw_choices)     
        return choices

    @property
    def is_multipart(self):
        return (self.type==FormField.FILE or self.type==FormField.SECURE_FILE or self.type==FormField.IMAGE)

    @property
    def input_type(self):
        return get_form_field_input_type_by_type(self.type)

    def get_form_field(self):
        return get_form_field_by_type(self.type, self, self.title, self.get_choices())

    @property
    def is_honeypot(self):
        if self.type == FormField.HONEYPOT_FIELD:
            return True
        return False

    def compress(self, raw_value):
    #PYTHON -> DATABASE
        if self.type == FormField.SELECT_MULTIPLE_CHECKBOXES or \
            self.type == FormField.SELECT_MULTIPLE_AUTOSUGGEST or \
            self.type == FormField.SELECT_MULTIPLE_HORIZONTAL or \
            self.type == FormField.SELECT_MULTIPLE_BUTTONS or \
            self.type == FormField.SELECT_MULTIPLE_IMAGES:

           
            if len(raw_value)==0:
                value = ''
            else:
                is_list = isinstance(raw_value, (list, tuple))

                if not is_list:
                    #SUSS OUT VALUE -- not sure what this does -nina
                    # if(len(raw_value)>0):
                    #     first_element = raw_value[0]
                    #     print 'frst element? %s'%(first_element)
                    #     if isinstance(first_element, (list)):
                    #         raw_value = first_element
                    unescaped = unescape(raw_value)
                    as_list = ast.literal_eval(unescaped.strip())
                    value = ', '.join(as_list)

                else:
                    value = ', '.join(raw_value)

            # print 'python->DB value %s -> %s'%(raw_value, value)
        else:
            value = raw_value
        
        return value

    def decompress(self, raw_value):
    #DATABASE -> PYTHON
        if self.type == FormField.SELECT_MULTIPLE_CHECKBOXES or \
            self.type == FormField.SELECT_MULTIPLE_AUTOSUGGEST or \
            self.type == FormField.SELECT_MULTIPLE_HORIZONTAL or \
            self.type == FormField.SELECT_MULTIPLE_BUTTONS or \
            self.type == FormField.SELECT_MULTIPLE_IMAGES:

            

            if isinstance(raw_value, basestring):
                if raw_value=='':
                    value = []
                else:
                    value = raw_value.split(', ')
                    # print 'decompress Raw value = %s; value = %s'%(raw_value, value)
            else:
                #already formatted
                value = raw_value

            # print 'DB->python value %s -> %s'%(raw_value, value)
        else:
            value = raw_value


        
        return value

    

    def generate_slug(self):
        starting_slug = '%s-%s'%(self.parent.slug, self.title)
        unique_slugify(self, starting_slug)
        return self.slug

    def __unicode__(self):
        return self.title

    class Meta:
        verbose_name_plural = "Form Fields"
        verbose_name = "Form Field"
        abstract = True



class FormEntry(VersionableAtom):

    form_schema = models.ForeignKey('form.Form', null=True, blank=True)

    NEW = 'new'
    READ = 'read'
    REPLIED = 'replied'
    ARCHIVED = 'archived'
    FORM_ENTRY_STATUS_CHOICES = (
        (NEW, _("New")),
        (READ, _("Read")),
        (REPLIED, _("Replied")),
        (ARCHIVED, _("Archived")),
    )

    status = models.CharField(choices=FORM_ENTRY_STATUS_CHOICES, 
        default=NEW, max_length = 255)

    tags = models.ManyToManyField('form.FormEntryTag', blank=True)

    def get_absolute_url(self):
        if self.form_schema.is_editable:
            return reverse('form_update_view', kwargs = {'path': self.form_schema.slug, 'pk':self.pk }) 
        return None

    def get_entry_by_slug(self, slug):
        try:
          return self.field_entry_model.objects.get(form_entry=self,form_field__slug=slug)
        except:
          return None

    def get_fields_with_entries(self):
        field_entry_hash = {}
        field_entries = self.get_entries()
        for field_entry in field_entries:
            field_entry_hash[field_entry.form_field.slug] = field_entry

        input_fields = self.form_schema.get_input_fields()

        #POPULATE KEY/VALUE PAIR:
        fields = []
        for field in input_fields:
            if field.slug in field_entry_hash:
                field_entry = field_entry_hash[field.slug]
                object = {'title':field.title,'value':field_entry.value,'entry':field_entry, 'field':field}
                fields.append(object)

        return fields


    def get_entries(self):
        return self.field_entry_model.objects.filter(form_entry=self).order_by('form_field__order')


    class Meta:
        abstract = True
        verbose_name = "Form Entry"
        verbose_name_plural = "Form Entries"


class FieldEntry(VersionableAtom):

    form_entry = models.ForeignKey('form.FormEntry', null=True, blank=True)
    form_field = models.ForeignKey('form.FormField', null=True, blank=True)
    
    value = models.TextField(null=True, blank=True)

    @property
    def decompressed_value(self):
    #DATABASE -> PYTHON
        return self.form_field.decompress(self.value)

    def get_compressed_value(self, value):
    #PYTHON -> DATABASE
        return self.form_field.compress(value)

    def get_rendered_value(self):
        if self.decompressed_value:
          if self.form_field.is_multipart:      
            file_url = "%s%s"%(settings.MEDIA_URL, self.value)
            if self.form_field.type==FormField.IMAGE:
                return '<img src="%s" >'%(file_url)
            elif self.form_field.type==FormField.FILE:
                return '<a href="%s" target="_blank">%s</a>'%(file_url, self.value)
            elif self.form_field.type==FormField.SECURE_FILE:
                duration = 120
                key_name = "%s/%s"%(settings.AWS_MEDIA_FOLDER, self.value)
                file_url = BaseSecureAtom.generate_link(settings.AWS_STORAGE_BUCKET_NAME_MEDIA_SECURE, key_name, duration)
                return '<a href="%s" target="_blank">%s</a><br /><br />This link will expire after %s seconds.</a>'%(file_url, self.value, duration)

            else:
                return file_url

          return self.value
        return ''

    
        
    
    class Meta:
        abstract = True
        verbose_name_plural = "Field Entries"
        verbose_name = "Field Entry"
        ordering = ['form_field__order']


class FormEntryStatus(CategoryMolecule):
    publish_by_default = True
    # default_template = 'media_tag'
    # item_classes = [Image, Media, SecureImage, SecureMedia]
    
    class Meta:
        verbose_name_plural = "Form Entry Statuses"
        verbose_name = "Form Entry Status"
        abstract = True

class FormEntryTag(TagMolecule):
    publish_by_default = True
    # default_template = 'media_tag'
    # item_classes = [Image, Media, SecureImage, SecureMedia]
    
    class Meta:
        verbose_name_plural = "Form Entry Tags"
        verbose_name = "Form Entry Tag"
        abstract = True        



def is_int(value):
    try: 
        int(value)
        return True
    except ValueError:
        return False

def unescape(value):
    from htmlentitydefs import name2codepoint
    # for some reason, python 2.5.2 doesn't have this one (apostrophe)
    name2codepoint['#39'] = 39

    return re.sub('&(%s);' % '|'.join(name2codepoint),
              lambda m: unichr(name2codepoint[m.group(1)]), value)




from os import path
from glob import glob
import os
import fnmatch
import re

from django.conf import settings
from django.core.cache import cache
try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model

try:
    # Django >= 1.8
    from django.template.loaders.app_directories import get_app_template_dirs
    app_template_dirs = get_app_template_dirs('templates')
except AttributeError:
    # Django <= 1.7
    from django.template.loaders.app_directories import app_template_dirs




def get_page_templates_raw(ignore_templates = None):

    if not ignore_templates:
        ignore_templates = ()

    files = []
    template_dirs = settings.TEMPLATES[0]['DIRS']
    for template_dir in template_dirs:
        for root, dirnames, filenames in os.walk(template_dir):
            for filename in fnmatch.filter(filenames, '*.html'):
                raw_file_name = os.path.join(root, filename)
                simple_path = raw_file_name.replace(template_dir+"/", "")
                files.append(str(simple_path))

    return files

def get_page_templates(ignore_templates = None):

    raw_templates = get_page_templates_raw(ignore_templates)
    output = []

    template_dirs = settings.TEMPLATES[0]['DIRS']
    for template_dir in template_dirs:
        
        def name(n):
            return n.replace(template_dir, '')\
                .replace('_', ' ')\
                .replace('.html', '')\
                .replace("-", " ")\
                .replace("/", " - ").title()
        output += [(i, name(i)) for i in raw_templates]

    return output


def get_all_templates():
   
    template_files = []
    for template_dir in (settings.TEMPLATES[0]['DIRS'] + app_template_dirs):
        for dir, dirnames, filenames in os.walk(template_dir):
            for filename in filenames:
                template_files.append(os.path.join(dir, filename))

    return template_files

def get_template_cache_key(template_pk_or_slug):
    return 'carbon_template_'+template_pk_or_slug

def get_template_by_pk_or_slug(template_pk_or_slug):

    found_template = None
    try:
        app_label = settings.TEMPLATE_MODEL.split('.')[0]
        object_name = settings.TEMPLATE_MODEL.split('.')[1]
        model = get_model(app_label, object_name)


        found_template = cache.get( get_template_cache_key(template_pk_or_slug) )
        
        if not found_template:

            if isinstance( template_pk_or_slug, ( int, long ) ):
                #try by pk
                try:
                    found_template = model.objects.get(pk=template_pk_or_slug)
                except:
                    pass

            if found_template == None:
                #try by slug
                try:
                    found_template = model.objects.get(slug=template_pk_or_slug)
                except:
                    pass

            #TRY WITH DB PREFIX
            prefix = 'template_'
            if found_template == None:
                #try by slug
                unprefixed_slug = template_pk_or_slug.replace(prefix, '')
                try:
                    found_template = model.objects.get(slug=unprefixed_slug)
                except:
                    pass

            cache.set(get_template_cache_key(template_pk_or_slug), found_template, settings.CACHE_DURATION)
            
    except:
        pass

    

    return found_template    


try:
    from django.apps import apps
    get_model = apps.get_model
except:
    from django.db.models.loading import get_model
from django.conf import settings
from django.dispatch import receiver
from django.db.models.signals import pre_delete, post_save



app_label = settings.CSS_RESOURCE_MODEL.split('.')[0]
object_name = settings.CSS_RESOURCE_MODEL.split('.')[1]
css_package_model = get_model(app_label, object_name)

app_label = settings.JS_RESOURCE_MODEL.split('.')[0]
object_name = settings.JS_RESOURCE_MODEL.split('.')[1]
js_package_model = get_model(app_label, object_name)

@receiver(pre_delete, sender=css_package_model, dispatch_uid='css_package_delete_signal')
def delete_css_resource(sender, instance, using, **kwargs): 
    instance.delete_downloaded_files()
    if instance.parent:
        instance.parent.request_render()

@receiver(pre_delete, sender=js_package_model, dispatch_uid='js_package_delete_signal')
def delete_js_resource(sender, instance, using, **kwargs):
    instance.delete_downloaded_files()
    if instance.parent:
        instance.parent.request_render()


@receiver(post_save, sender=css_package_model, dispatch_uid='css_package_save_signal')
def save_css_resource(sender, instance, created, **kwargs):
    if created:
        if instance.parent:
            instance.parent.request_render()

@receiver(post_save, sender=js_package_model, dispatch_uid='js_package_save_signal')
def save_js_resource(sender, instance, created, **kwargs):
    if created:
        if instance.parent:
            instance.parent.request_render()
 


from django.contrib import admin
from django.utils.safestring import mark_safe

from .content import *

class BaseTagAdmin(BaseContentAdmin):

    def admin_hierarchy(self, obj):
        return mark_safe(obj.admin_hierarchy)

    autocomplete_lookup_fields = {
        'fk': ('template','image'),
    }
    raw_id_fields = ( 'template','image')
    
    
    list_display = ( "admin_hierarchy", "path",  "title", "publication_status",)
    list_display_links = ( "admin_hierarchy", "path", "title",)
    list_filter = (
            "publication_status", "created_by", "modified_by", 
            'published_by','is_searchable','in_sitemap',
            'sitemap_changefreq','sitemap_priority','noindex','nofollow',
            'shareable','social_share_type')
    ordering = ("hierarchy",)



    prepopulated_fields = {"slug": ("title",)}
    
    readonly_fields = (
        "version", "created_date", "created_by", "modified_date", "modified_by",
         "path", "path_generated", "uuid", 'image_preview', 'edit_template'
    )
    


    core_fields = (
        ('title','slug'),
        'content',
        'synopsis',
        ('image_preview','image')

    )

    publication_fields = (
        ('publication_status'),
        ('publication_date', 'published_by'),
        ('publish_on_date', 'expire_on_date'),
        'facebook_author_id',
        'twitter_author_id',
        'google_author_id'
    )
   
    path_fields = (
        ('template', 'edit_template'),
        ('path', 'uuid',),
        ('path_generated', 'path_override'),
        ('temporary_redirect', 'permanent_redirect'),
        'order'

    )

    seo_fields = BaseContentAdmin.seo_fields
    social_fields =  (
        ('shareable','social_share_type'),
        'tiny_url',
        'social_share_image',        
    )
    meta_fields = BaseVersionableAdmin.meta_fields


    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Path", {
            'fields': path_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Publication", {
            'fields': publication_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        
        ("Search Engine Optimization", {
            'fields': seo_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Social Integration", {
            'fields': social_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )

class BaseSimplifiedTagAdmin(BaseContentAdmin):

    def admin_hierarchy(self, obj):
        return mark_safe(obj.admin_hierarchy)
    

    list_display = ( "admin_hierarchy", "path",  "title", "publication_status",)
    list_display_links = ( "admin_hierarchy", "path", "title",)
    list_filter = (
        "publication_status", "created_by", "modified_by", 
    )
    ordering = ("hierarchy",)



    prepopulated_fields = {"slug": ("title",)}
    
    readonly_fields = (
        "version", "created_date", "created_by", "modified_date", "modified_by",
    )
    
    


    core_fields = (
        ('title','slug'),
    )


    meta_fields = BaseVersionableAdmin.meta_fields

    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )    

class BaseCategoryAdmin(BaseTagAdmin):

    autocomplete_lookup_fields = {
        'fk': ('parent','template','image'),
    }
    raw_id_fields = ( 'parent','template','image')

    core_fields = BaseTagAdmin.core_fields
    core_fields_list = list(core_fields)
    core_fields_list.insert(0, ('edit_parent','parent'))
    core_fields = tuple(core_fields_list)

    list_filter = BaseTagAdmin.list_filter
    list_filter_list = list(list_filter)
    list_filter_list.insert(0, 'parent')
    list_filter = tuple(list_filter_list)

    path_fields = BaseTagAdmin.path_fields
    publication_fields = BaseTagAdmin.publication_fields
    seo_fields = BaseTagAdmin.seo_fields
    social_fields = BaseTagAdmin.social_fields
    meta_fields = BaseTagAdmin.meta_fields

    readonly_fields = BaseTagAdmin.readonly_fields + ('edit_parent',)

    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Path", {
            'fields': path_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Publication", {
            'fields': publication_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        
        ("Search Engine Optimization", {
            'fields': seo_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Social Integration", {
            'fields': social_fields,
            'classes': ( 'grp-collapse grp-closed', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )

class BaseSimplifiedCategoryAdmin(BaseSimplifiedTagAdmin):

    autocomplete_lookup_fields = {
        'fk': ('parent',),
    }
    raw_id_fields = ( 'parent',)

    core_fields = BaseSimplifiedTagAdmin.core_fields
    core_fields_list = list(core_fields)
    core_fields_list.insert(0, ('edit_parent','parent'))
    core_fields = tuple(core_fields_list)

    list_filter = BaseSimplifiedTagAdmin.list_filter
    list_filter_list = list(list_filter)
    list_filter_list.insert(0, 'parent')
    list_filter = tuple(list_filter_list)

    
    meta_fields = BaseSimplifiedTagAdmin.meta_fields

    readonly_fields = BaseSimplifiedTagAdmin.readonly_fields + ('edit_parent',)

    fieldsets = (
        ("Main Body", {
            'fields': core_fields,
            'classes': ( 'grp-collapse grp-open', )
        }),
        ("Meta", {
            'fields': meta_fields,
            'classes': ( 'grp-collapse grp-closed', )
        })
    )


import os
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError

from carbon.utils.template import get_page_templates, get_page_templates_raw

from ...models import Template


class Command(BaseCommand):
    args = ''
    help = ''

    def handle(self, *args, **options):

        #Create file templates if any are missing
        
        template_dirs = settings.TEMPLATES[0]['DIRS']

        for template_dir in template_dirs:
            for root, dirs, files in os.walk(template_dir):
                for file in files:
                    
                    template, created = Template.objects.get_or_create(file_template = file)
                    if not template.title or 'untitled' in template.title.lower():
                        template.title = u'File Template %s'%(file)
                        template.save()
                    




from django.db import models



class PublishableQueryset(models.query.QuerySet):

    DRAFT = 10
    REVIEW = 20
    PUBLISHED = 100
    UNPUBLISHED = 40

    def published(self):
        return self.filter(publication_status = PublishableQueryset.PUBLISHED)

    def drafts(self):
        return self.filter(publication_status__lte = PublishableQueryset.PUBLISHED)

    def unpublished(self):
        return self.filter(publication_status = PublishableQueryset.UNPUBLISHED)


class PublishableManager(models.Manager):
    def get_queryset(self):
        return PublishableQueryset(self.model, using=self._db)

    def published(self):
        # WARNING: Just because something is set to be published doesn't mean
        # it should be published -- an item could be expired or have some
        # other limitation
        return self.get_queryset().published()

    def drafts(self):
        return self.get_queryset().drafts()
    
    def unpublished(self):
        return self.get_queryset().unpublished()

